@@ -1,7 +1,6 @@
 """Urls for Django Rest Framework."""
 
 from django.urls import include, path
-
 from rest_framework import routers
 
 from .flow.views import (
@@ -17,7 +16,7 @@ from .flow.views import (
     StorageViewSet,
 )
 from .observers.views import BackgroundTaksViewSet
-from .storage.views import UploadConfig
+from .storage.views import UploadConfig, UploadCredentials
 
 api_router = routers.DefaultRouter(trailing_slash=False)
 api_router.register(r"collection", CollectionViewSet)
@@ -27,7 +26,6 @@ api_router.register(r"entity", EntityViewSet)
 api_router.register(r"relation", RelationViewSet)
 api_router.register(r"descriptorschema", DescriptorSchemaViewSet)
 api_router.register(r"storage", StorageViewSet)
-api_router.register(r"upload_config", UploadConfig, basename="upload_config")
 api_router.register(r"task", BackgroundTaksViewSet, basename="backgroundtask")
 api_router.register(r"annotation_field", AnnotationFieldViewSet)
 api_router.register(r"annotation_preset", AnnotationPresetViewSet)
@@ -36,4 +34,8 @@ api_router.register(r"annotation_value", AnnotationValueViewSet)
 
 urlpatterns = [
     path("api/", include((api_router.urls, "resolwe-api"), namespace="resolwe-api")),
+    path(
+        "api/upload/credentials", UploadCredentials.as_view(), name="upload_credentials"
+    ),
+    path("api/upload/config", UploadConfig.as_view(), name="upload_config"),
 ]
