@@ -53,9 +53,9 @@ class AccessLogger:
     def __init__(self):
         """Initialize."""
         #        self._access =
-        self._access: Dict[
-            ContentType, Dict[ObjectId, Dict[AccessType, Fields]]
-        ] = defaultdict(lambda: defaultdict(lambda: defaultdict(set)))
+        self._access: Dict[ContentType, Dict[ObjectId, Dict[AccessType, Fields]]] = (
+            defaultdict(lambda: defaultdict(lambda: defaultdict(set)))
+        )
 
     def clear(self):
         """Clear all accesses."""
@@ -156,6 +156,7 @@ class AuditManager:
     def reset(self):
         """Clear all stored data."""
         self._access_loger.clear()
+        self._custom_messages.clear()
 
     @staticmethod
     def global_instance() -> "AuditManager":
