@@ -1,13 +1,11 @@
 """An execution engine for Python processes."""
-import os
+
+from pathlib import Path
 
 from resolwe.flow.execution_engines.base import BaseExecutionEngine
+from resolwe.flow.executors.constants import BOOTSTRAP_PYTHON_RUNTIME
 from resolwe.process.parser import SafeParser
 
-PYTHON_RUNTIME_DIRNAME = "python_runtime"
-PYTHON_RUNTIME_ROOT = "/"
-PYTHON_RUNTIME_VOLUME = os.path.join(PYTHON_RUNTIME_ROOT, PYTHON_RUNTIME_DIRNAME)
-
 
 class ExecutionEngine(BaseExecutionEngine):
     """An execution engine that outputs bash programs."""
@@ -23,16 +21,12 @@ class ExecutionEngine(BaseExecutionEngine):
         """
         if not path.lower().endswith(".py"):
             return []
-        parser = SafeParser(open(path).read())
-        processes = parser.parse()
-        return [process.to_schema() for process in processes]
+        with open(path) as handle:
+            parser = SafeParser(handle.read())
+            processes = parser.parse()
+            return [process.to_schema() for process in processes]
 
     def evaluate(self, data):
         """Evaluate the code needed to compute a given Data object."""
-        return 'PYTHONPATH="{runtime}" python3 -u -m resolwe.process'.format(
-            runtime=PYTHON_RUNTIME_VOLUME
-        )
-
-    def prepare_volumes(self):
-        """Mount additional volumes."""
-        return {PYTHON_RUNTIME_DIRNAME: PYTHON_RUNTIME_VOLUME}
+        bootstrap_filename = Path("/") / BOOTSTRAP_PYTHON_RUNTIME
+        return f"python3 {bootstrap_filename}"
