@@ -10,6 +10,7 @@ Framework for the manager-resident executor preparation facilities.
     :members:
 
 """
+
 import logging
 import os
 
@@ -49,16 +50,18 @@ class BaseFlowExecutorPreparer:
         """
         pass
 
-    def get_tools_paths(self):
+    def get_tools_paths(self, from_applications=False):
         """Get tools' paths."""
-        if settings.DEBUG or is_testing():
+        if settings.DEBUG or is_testing() or from_applications:
             return list(get_apps_tools().values())
 
         else:
             tools_root = storage_settings.FLOW_VOLUMES["tools"]["config"]["path"]
+        try:
             subdirs = next(os.walk(tools_root))[1]
-
             return [os.path.join(tools_root, sdir) for sdir in subdirs]
+        except StopIteration:
+            return []
 
     def post_register_hook(self, verbosity=1):
         """Run hook after the 'register' management command finishes.
