@@ -34,7 +34,7 @@ async def async_zmq_send_data(
 
 async def async_zmq_receive_data(
     reader: zmq.asyncio.Socket,
-) -> Optional[Tuple[PeerIdentity, Any]]:
+) -> Tuple[PeerIdentity, Any, Optional[bytes]]:
     """Receive data from the reader.
 
     The data is expected to be bytes-encoded JSON representation of a Python object.
@@ -45,13 +45,16 @@ async def async_zmq_receive_data(
 
     :raises zmq.ZMQError: on receive error.
     """
+    user_id = None
     if reader.socket_type == zmq.DEALER:
-        identity = reader.getsockopt(zmq.IDENTITY)
-        message = await reader.recv()
+        identity = str(reader.getsockopt(zmq.IDENTITY)).encode()
+        message = await reader.recv(copy=False)
     else:
-        identity, message = await reader.recv_multipart()
-    decoded = json.loads(message.decode())
-    return (identity, decoded)
+        received_identity, message = await reader.recv_multipart(copy=False)
+        identity = received_identity.bytes
+        user_id = str(message["User-Id"]).encode()
+    decoded = json.loads(message.bytes.decode())
+    return (identity, decoded, user_id)
 
 
 class ZMQCommunicator(BaseCommunicator):
