@@ -9,26 +9,31 @@ Flow Filters
 import re
 import types
 from copy import deepcopy
-from functools import partial
-from typing import Callable, Union
-
-from django_filters import rest_framework as filters
-from django_filters.constants import EMPTY_VALUES
-from django_filters.filterset import FilterSetMetaclass
-from versionfield import VersionField
+from dataclasses import dataclass
+from functools import partial, reduce
+from typing import Any, Optional, Union
 
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
 from django.contrib.postgres.search import SearchQuery, SearchRank
-from django.core.exceptions import ValidationError
-from django.db.models import Count, F, ForeignKey, Q, Subquery
+from django.core.exceptions import FieldDoesNotExist, ValidationError
+from django.db import connection
+from django.db.models import F, ForeignKey, Model, OuterRef, Q, Subquery
+from django.db.models.expressions import RawSQL
 from django.db.models.query import QuerySet
-
+from django_filters import rest_framework as filters
+from django_filters.constants import EMPTY_VALUES
+from django_filters.filterset import FilterSetMetaclass
 from rest_framework import fields
 from rest_framework.filters import OrderingFilter as DrfOrderingFilter
 
 from resolwe.composer import composer
-from resolwe.permissions.models import Permission, get_anonymous_user
+from resolwe.flow.models.fields import VersionField
+from resolwe.permissions.models import (
+    Permission,
+    PermissionInterface,
+    get_anonymous_user,
+)
 
 from .models import (
     AnnotationField,
@@ -43,6 +48,11 @@ from .models import (
 )
 from .models.annotations import AnnotationType
 
+# The key used in the full-text search filter.
+FULL_TEXT_SEARCH_KEY = "text"
+# The actual database fields used for full-text search for every supported entity.
+FULL_TEXT_SEARCH_FIELD = {"Entity": "search", "Data": "search", "Collection": "search"}
+
 RELATED_LOOKUPS = [
     "exact",
     "in",
@@ -99,6 +109,8 @@ class CheckQueryParamsMixin:
             "limit",
             "offset",
             "ordering",
+            # Used for full-text search.
+            "text",
         )
 
     def validate_query_params(self):
@@ -122,21 +134,6 @@ class CheckQueryParamsMixin:
         return super().is_valid()
 
 
-class TextFilterMixin:
-    """Mixin for full-text filtering."""
-
-    def filter_text(self, queryset: QuerySet, name: str, value: str):
-        """Full-text search."""
-        query = SearchQuery(value, config="simple")
-        return (
-            queryset.filter(**{name: query})
-            # This assumes that field is already a TextSearch vector and thus
-            # doesn't need to be transformed. To achieve that F function is
-            # required.
-            .annotate(rank=SearchRank(F(name), query)).order_by("-rank")
-        )
-
-
 class UserFilterMixin:
     """Mixin for filtering by contributors and owners."""
 
@@ -181,7 +178,9 @@ class UserFilterMixin:
         """Filter queryset by owner's name."""
         return queryset.filter(contributor__in=self._get_user_subquery(value))
 
-    def filter_for_user(self, queryset: QuerySet, name: str, value: str):
+    def filter_for_user(
+        self, queryset: QuerySet[PermissionInterface], name: str, value: str
+    ) -> QuerySet[PermissionInterface]:
         """Filter queryset by permissions."""
         user = self.request.user
         return queryset.filter_for_user(user, Permission.from_name(value))
@@ -235,18 +234,115 @@ class ResolweFilterMetaclass(FilterSetMetaclass):
 
     """
 
-    def __new__(mcs, name, bases, namespace):
-        """Inject extensions into the filter."""
-        class_path = "{}.{}".format(namespace["__module__"], name)
-        for extension in composer.get_extensions(class_path):
-            for name in dir(extension):
-                if name.startswith("__"):
-                    continue
-                namespace[name] = getattr(extension, name)
+    def __new__(
+        mcs: type[object],
+        name: str,
+        bases: tuple[type[object]],
+        namespace: dict[str, Any],
+    ):
+        """Inject filters."""
 
+        def inject_composer_extensions(namespace: dict[str, Any], name: str):
+            """Inject filters from composer extensions."""
+            class_path = "{}.{}".format(namespace["__module__"], name)
+            for extension in composer.get_extensions(class_path):
+                for name in dir(extension):
+                    if name.startswith("__"):
+                        continue
+                    namespace[name] = getattr(extension, name)
+
+        def get_related_model_from_path(related_path: str) -> type[Model]:
+            """Get the related model from the path."""
+            return reduce(
+                lambda Model, part: Model._meta.get_field(part).related_model,
+                related_path.split("__"),
+                namespace["Meta"].model,
+            )
+
+        def add_filters_with_permissions(
+            prefix: str, related_path: str, base_filter: filters.FilterSet
+        ):
+            """Add filters on related objects with permissions.
+
+            All the filters belonging to the BaseFilter are added with the given prefix.
+            """
+
+            def filter_permissions(
+                self,
+                qs: QuerySet,
+                value: str,
+                original_filter: filters.Filter,
+                original_model: Model,
+            ):
+                """Apply the filter and respect permissions."""
+
+                # Do not filter when value is empty. At least one of the values must be
+                # non-empty since form in the AnnotationValueFilter class requires it.
+                if value in EMPTY_VALUES:
+                    return qs
+
+                visible_objects = list(
+                    original_filter.filter(original_model.objects.all(), value)
+                    .filter_for_user(self.parent.request.user)
+                    .values_list("pk", flat=True)
+                )
+                return qs.filter(**{f"{related_path}__in": visible_objects})
+
+            # Add all filters from EntityFilter to namespaces before creating class.
+            BaseModel = get_related_model_from_path(related_path)
+            for filter_name, filter in base_filter.get_filters().items():
+                new_filter_name = f"{prefix}__{filter_name}"
+                if filter_name == "id" or filter_name.startswith("id__"):
+                    new_filter_name = f"{prefix}{filter_name[2:]}"
+
+                filter_copy = deepcopy(filter)
+                filter_copy.field_name = f"{prefix}__{filter_copy.field_name}"
+                filter_method = partial(
+                    filter_permissions, original_filter=filter, original_model=BaseModel
+                )
+                # Bind the new_filter to filter instance and set it as new filter.
+                filter_copy.filter = types.MethodType(filter_method, filter_copy)
+                namespace[new_filter_name] = filter_copy
+                # If filter uses a method, add it to the namespace as well.
+                if filter_copy.method is not None:
+                    namespace[filter_copy.method] = deepcopy(
+                        getattr(base_filter, filter_copy.method)
+                    )
+
+        def inject_related_permissions(namespace: dict):
+            """Parse the namespace and add filters with permissions."""
+            for prefix, value in list(namespace.items()):
+                if isinstance(value, FilterRelatedWithPermissions):
+                    namespace.pop(prefix)
+                    add_filters_with_permissions(
+                        prefix, value.related_path or prefix, value.BaseFilter
+                    )
+
+        inject_related_permissions(namespace)
+        inject_composer_extensions(namespace, name)
         return super().__new__(mcs, name, bases, namespace)
 
 
+@dataclass
+class FilterRelatedWithPermissions:
+    """Base class for filters with permissions.
+
+    The class using this must have FilterRelatedWithPermissionsMeta as its meta class.
+    The filter is then defined as:
+
+    prefix = FilterRelatedWithPermissions(BaseFilter, related_path="related__path")
+
+    All the filters from the BaseFilter (with the given prefix) are added and
+    permissions on the related objects are respected.
+
+    The optional parameter `related__path` is used to specify the path of the related
+    object (if different from the prefix).
+    """
+
+    BaseFilter: type[filters.FilterSet]
+    related_path: Optional[str] = None
+
+
 class BaseResolweFilter(
     CheckQueryParamsMixin, filters.FilterSet, metaclass=ResolweFilterMetaclass
 ):
@@ -282,7 +378,11 @@ class DescriptorSchemaFilter(BaseResolweFilter):
         model = DescriptorSchema
 
 
-class BaseCollectionFilter(TextFilterMixin, UserFilterMixin, BaseResolweFilter):
+class CharInFilter(filters.BaseInFilter, filters.CharFilter):
+    """Basic filter for CharField with 'in' lookup."""
+
+
+class BaseCollectionFilter(UserFilterMixin, BaseResolweFilter):
     """Base filter for Collection and Entity endpoints."""
 
     contributor_name = filters.CharFilter(method="filter_contributor_name")
@@ -294,7 +394,8 @@ class BaseCollectionFilter(TextFilterMixin, UserFilterMixin, BaseResolweFilter):
     )
     permission = filters.CharFilter(method="filter_for_user")
     tags = TagsFilter()
-    text = filters.CharFilter(field_name="search", method="filter_text")
+    status = filters.CharFilter(field_name="status")
+    status__in = CharInFilter(field_name="status", lookup_expr="in")
 
     class Meta(BaseResolweFilter.Meta):
         """Filter configuration."""
@@ -303,7 +404,6 @@ class BaseCollectionFilter(TextFilterMixin, UserFilterMixin, BaseResolweFilter):
             **BaseResolweFilter.Meta.fields,
             **{
                 "description": TEXT_LOOKUPS[:],
-                "descriptor_schema": ["exact"],
             },
         }
 
@@ -311,31 +411,22 @@ class BaseCollectionFilter(TextFilterMixin, UserFilterMixin, BaseResolweFilter):
 class CollectionFilter(BaseCollectionFilter):
     """Filter the Collection endpoint."""
 
-    entity_count = filters.NumberFilter(
-        method="count_entities", field_name="entity__count"
-    )
-    entity_count__gt = filters.NumberFilter(
-        method="count_entities", field_name="entity__count__gt"
-    )
-    entity_count__lt = filters.NumberFilter(
-        method="count_entities", field_name="entity__count__lt"
-    )
-    entity_count__gte = filters.NumberFilter(
-        method="count_entities", field_name="entity__count__gte"
-    )
-    entity_count__lte = filters.NumberFilter(
-        method="count_entities", field_name="entity__count__lte"
-    )
+    entity_count = filters.NumberFilter(field_name="entity_count", lookup_expr="exact")
+    entity_count__gt = filters.NumberFilter("entity_count", "gt")
+    entity_count__lt = filters.NumberFilter("entity_count", "lt")
+    entity_count__gte = filters.NumberFilter("entity_count", "gte")
+    entity_count__lte = filters.NumberFilter("entity_count", "lte")
 
     class Meta(BaseCollectionFilter.Meta):
         """Filter configuration."""
 
         model = Collection
-
-    def count_entities(self, queryset: QuerySet, name: str, value: str):
-        """Filter by the number of associated entities."""
-
-        return queryset.annotate(Count("entity")).filter(**{name: value})
+        fields = {
+            **BaseCollectionFilter.Meta.fields,
+            **{
+                "descriptor_schema": ["exact"],
+            },
+        }
 
 
 class EntityFilter(BaseCollectionFilter):
@@ -439,14 +530,13 @@ class ProcessFilter(BaseResolweFilter):
         }
 
 
-class DataFilter(TextFilterMixin, UserFilterMixin, BaseResolweFilter):
+class DataFilter(UserFilterMixin, BaseResolweFilter):
     """Filter the Data endpoint."""
 
     contributor_name = filters.CharFilter(method="filter_contributor_name")
     owners = filters.CharFilter(method="filter_owners")
     owners_name = filters.CharFilter(method="filter_owners_name")
     tags = TagsFilter()
-    text = filters.CharFilter(field_name="search", method="filter_text")
     type = filters.CharFilter(field_name="process__type", lookup_expr="startswith")
     type__exact = filters.CharFilter(field_name="process__type", lookup_expr="exact")
     relation_id = filters.NumberFilter(
@@ -517,79 +607,190 @@ class OrderingFilter(DrfOrderingFilter):
     applies ordering by ranking.
     """
 
-    def get_ordering(self, request, queryset, view):
-        """Return name of the filed by which results should be ordered.
+    @property
+    def _annotation_handlers(self):
+        """Return a list of custom annotation handlers."""
+        return (self._annotate_annotations,)
 
-        Ordering is skipped if ``None`` is returned.
+    def _annotate_annotations(self, queryset, ordering):
+        """Annotate queryset with annotations.
+
+        The annotations are only applied if ordering by annotations is requested.
         """
-        params = request.query_params.get(self.ordering_param)
-        if params:
-            fields = [param.strip() for param in params.split(",")]
-            ordering = self.remove_invalid_fields(queryset, fields, view, request)
-            if ordering:
-                return ordering
+        if not issubclass(queryset.model, Entity):
+            return queryset
 
-        # Skip default ordering as full-text search applies ordering by rank.
-        if request.query_params.get("text"):
+        regex = re.compile(r"-?annotations_order_(?P<field_id>\d+)")
+        for field in ordering:
+            if not isinstance(field, str):
+                continue
+            if match := regex.match(field):
+                field_id = match.group("field_id")
+                # Always filter by user-visible attribute annotation_label.
+                annotation_value = AnnotationValue.objects.filter(
+                    entity_id=OuterRef("pk"), field_id=field_id
+                ).values("_value__label")
+                annotate = {f"annotations_order_{field_id}": Subquery(annotation_value)}
+                queryset = queryset.annotate(**annotate)
+        return queryset
+
+    def _annotate_queryset(self, queryset, ordering):
+        """Annotate queryset with ordering fields.
+
+        Some ordering field require additional annotations to be present.
+        """
+        for handler in self._annotation_handlers:
+            queryset = handler(queryset, ordering)
+        return queryset
+
+    def filter_queryset(self, request, queryset, view):
+        """Return ordered queryset."""
+        if ordering := self.get_ordering(request, queryset, view):
+            queryset = self._annotate_queryset(queryset, ordering).order_by(*ordering)
+        return queryset
+
+    def _remove_invalid_annotation_field(self, queryset, field, view, request):
+        """Check if the given sorting field is annotations field.
+
+        The format of the ordering field is annotations__field_id, possibly prefixed by
+        a minus sign. The minus sign indicates descending order.
+        """
+        if re.match(r"-?annotations__(?P<field_id>\d+)", field):
+            return field.replace("__", "_order_")
+
+    def _remove_invalid_json_field(self, queryset, field, view, request):
+        """Remove invalid JSON field.
+
+        Either return None or a ordering expression.
+        """
+        # When a dot is present in the order field, treat it as a JSON path.
+        if "." not in field:
             return None
 
-        return self.get_default_ordering(view)
+        path = field.split(".")
+        base_field = path[0]
+        if len(path) < 2:
+            return None
+        if not super().remove_invalid_fields(queryset, [base_field], view, request):
+            return None
 
+        reverse = False
+        if base_field[0] == "-":
+            base_field = base_field[1:]
+            reverse = True
 
-class AnnotationValueFieldMetaclass(ResolweFilterMetaclass):
-    """Add all entity filters prefixed with 'entity'."""
-
-    def __new__(mcs, name, bases, namespace):
-        """Inject extensions into the filter."""
-
-        def filter_permissions(
-            self,
-            qs: QuerySet,
-            value: str,
-            original_filter: Callable[[QuerySet, str], QuerySet],
-        ):
-            """Respect permissions on entities."""
-            # Do not filter when value is empty. At least one of the values must be
-            # non-empty since form in the AnnotationValueFilter class requires it.
-            if value in EMPTY_VALUES:
-                return qs
-            qs = original_filter(qs, value)
-            user = self.parent.request.user
-            entity_ids = qs.values_list(f"{entity_path}_id")
-            visible_entities = Entity.objects.filter(id__in=entity_ids).filter_for_user(
-                user
+        # Resolve base field.
+        try:
+            quote_name = connection.ops.quote_name
+            model_meta = queryset.model._meta
+            base_field = "{}.{}".format(
+                quote_name(model_meta.db_table),
+                quote_name(model_meta.get_field(base_field).column),
             )
-            return qs.filter(**{f"{entity_path}__in": visible_entities})
+        except FieldDoesNotExist:
+            return None
 
-        entity_path = {
-            "AnnotationValueFilter": "entity",
-            "AnnotationFieldFilter": "values__entity",
-        }[name]
-        # Add all filters from EntityFilter to namespaces before creating class.
-        for filter_name, filter in EntityFilter.get_filters().items():
-            new_filter_name = f"entity__{filter_name}"
-            if filter_name == "id" or filter_name.startswith("id__"):
-                new_filter_name = "entity" + filter_name[2:]
-            filter = deepcopy(filter)
-            filter.field_name = f"{entity_path}__{filter.field_name}"
-            new_filter = partial(filter_permissions, original_filter=filter.filter)
-            # Bind the new_filter to filter instance and set it as new filter.
-            filter.filter = types.MethodType(new_filter, filter)
-            namespace[new_filter_name] = filter
-            # If filter uses a method, add it to the namespace as well.
-            if filter.method is not None:
-                namespace[filter.method] = deepcopy(
-                    getattr(EntityFilter, filter.method)
+        placeholders = "->".join(["%s"] * (len(path) - 2))
+        # The last placeholder should be accessed via the ->> operator.
+        if placeholders:
+            placeholders = "->{}->>%s".format(placeholders)
+        else:
+            placeholders = "->>%s"
+
+        expression = RawSQL(
+            # We can use base_field here directly because we've resolved it via Django ORM.
+            "{}{}".format(base_field, placeholders),
+            params=path[1:],
+        )
+        if reverse:
+            expression = expression.desc()
+
+        return expression
+
+    @property
+    def _invalid_field_handlers(self):
+        """Return a list of custom invalid field handlers."""
+        return (
+            self._remove_invalid_annotation_field,
+            self._remove_invalid_json_field,
+        )
+
+    def _remove_invalid_field(self, queryset, field, view, request):
+        """Remove invalid fields from the ordering.
+
+        Returns the ordering expression if the field is valid, otherwise None.
+        """
+        arguments = (queryset, field, view, request)
+        for handler in self._invalid_field_handlers:
+            if field := handler(*arguments):
+                return field
+        return None
+
+    def remove_invalid_fields(self, queryset, fields, view, request):
+        """Remove invalid fields and preserve sort order.
+
+        First check for custom ordering fields and then for the default ones.
+        """
+        order_fields = []
+        base_order_fields = super().remove_invalid_fields(
+            queryset, fields, view, request
+        )
+        for field in fields:
+            if field in base_order_fields:
+                order_fields.append(field)
+            elif expression := self._remove_invalid_field(
+                queryset, field, view, request
+            ):
+                order_fields.append(expression)
+            else:
+                raise ValidationError(
+                    "Ordering by field '{}' is not supported.".format(field)
+                )
+        return order_fields
+
+
+class FullTextSearchFilter(DrfOrderingFilter):
+    """Full text search filter.
+
+    Filter and order the queryset by the full text search rank.
+    """
+
+    def filter_queryset(self, request, queryset, view):
+        """Return ordered queryset.
+
+        If full text search is used and ordering by some other field is requested, raise
+        an error.
+        """
+        if FULL_TEXT_SEARCH_KEY in view.request.query_params:
+
+            order_fields = request.query_params.get(self.ordering_param)
+            if FULL_TEXT_SEARCH_KEY in view.request.query_params and order_fields:
+                raise ValidationError(
+                    f"Ordering by full text search rank and other fields ({order_fields}) "
+                    f"is not supported."
                 )
 
-        # Create class with added filters.
-        klass = ResolweFilterMetaclass.__new__(mcs, name, bases, namespace)
-        return klass
+            model_name = queryset.model.__name__
+            field_name = FULL_TEXT_SEARCH_FIELD.get(model_name)
+            if field_name is None:
+                raise ValidationError(
+                    f"Full text search is not supported for the model {model_name}."
+                )
+            value = view.request.query_params[FULL_TEXT_SEARCH_KEY]
+            query = SearchQuery(value, config="simple_unaccent")
+            return (
+                queryset.filter(**{field_name: query})
+                .annotate(rank=SearchRank(F(field_name), query))
+                .order_by("-rank")
+            )
+        return queryset
 
 
-class AnnotationFieldFilter(BaseResolweFilter, metaclass=AnnotationValueFieldMetaclass):
+class AnnotationFieldFilter(BaseResolweFilter):
     """Filter the AnnotationField endpoint."""
 
+    entity = FilterRelatedWithPermissions(EntityFilter, related_path="values__entity")
+
     @classmethod
     def filter_for_field(cls, field, field_name, lookup_expr=None):
         """Add permission check for collections lookups.
@@ -669,9 +870,10 @@ class AnnotationPresetFilter(BaseResolweFilter):
         }
 
 
-class AnnotationValueFilter(BaseResolweFilter, metaclass=AnnotationValueFieldMetaclass):
+class AnnotationValueFilter(BaseResolweFilter):
     """Filter the AnnotationValue endpoint."""
 
+    entity = FilterRelatedWithPermissions(EntityFilter)
     label = filters.CharFilter(method="filter_by_label")
 
     def filter_by_label(self, queryset: QuerySet, name: str, value: str):
@@ -689,6 +891,7 @@ class AnnotationValueFilter(BaseResolweFilter, metaclass=AnnotationValueFieldMet
                 "field__name": TEXT_LOOKUPS[:],
                 "field__label": TEXT_LOOKUPS[:],
                 "field__group__name": TEXT_LOOKUPS[:],
-                "modified": DATE_LOOKUPS[:],
+                "created": DATE_LOOKUPS[:],
+                "contributor": ["exact", "in"],
             },
         }
