@@ -8,11 +8,10 @@ Register Processes
 
 import os
 import re
+from contextlib import suppress
 
 import jsonschema
 import yaml
-from versionfield.utils import convert_version_string_to_int
-
 from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.core.exceptions import ValidationError
@@ -24,6 +23,7 @@ from resolwe.flow.finders import get_finders
 from resolwe.flow.managers import manager
 from resolwe.flow.models import DescriptorSchema, Process
 from resolwe.flow.models.base import VERSION_NUMBER_BITS
+from resolwe.flow.models.fields import convert_version_string_to_int
 from resolwe.flow.models.utils import validate_schema, validation_schema
 from resolwe.flow.utils import dict_dot, iterate_schema
 from resolwe.permissions.utils import assign_contributor_permissions, copy_permissions
@@ -184,22 +184,10 @@ class Command(BaseCommand):
                     continue
 
                 p["entity_type"] = p["entity"]["type"]
-                p["entity_descriptor_schema"] = p["entity"].get(
-                    "descriptor_schema", p["entity_type"]
-                )
                 p["entity_input"] = p["entity"].get("input", None)
                 p["entity_always_create"] = p["entity"].get("always_create", False)
                 p.pop("entity")
 
-                if not DescriptorSchema.objects.filter(
-                    slug=p["entity_descriptor_schema"]
-                ).exists():
-                    self.stderr.write(
-                        "Skip processor {}: Unknown descriptor schema '{}' used in 'entity' "
-                        "field.".format(p["slug"], p["entity_descriptor_schema"])
-                    )
-                    continue
-
             if "persistence" in p:
                 persistence_mapping = {
                     "RAW": Process.PERSISTENCE_RAW,
@@ -266,11 +254,9 @@ class Command(BaseCommand):
 
             version = p["version"]
             int_version = convert_version_string_to_int(version, VERSION_NUMBER_BITS)
-
-            # `latest version` is returned as `int` so it has to be compared to `int_version`
-            latest_version = Process.objects.filter(slug=slug).aggregate(
-                Max("version")
-            )["version__max"]
+            latest_version = None
+            with suppress(Process.DoesNotExist):
+                latest_version = Process.objects.get(slug=slug).version
             if latest_version is not None and latest_version > int_version:
                 self.stderr.write(
                     "Skip processor {}: newer version installed".format(slug)
@@ -283,7 +269,7 @@ class Command(BaseCommand):
             else:
                 previous_process = None
 
-            process_query = Process.objects.filter(slug=slug, version=version)
+            process_query = Process.all_objects.filter(slug=slug, version=version)
             if process_query.exists():
                 is_active = process_query.values_list("is_active", flat=True).get()
                 if not is_active:
@@ -397,7 +383,7 @@ class Command(BaseCommand):
         process_slugs = set(ps["slug"] for ps in process_schemas)
 
         # Processes that are in DB but not in the code
-        retired_processes = Process.objects.filter(~Q(slug__in=process_slugs))
+        retired_processes = Process.all_objects.filter(~Q(slug__in=process_slugs))
 
         # Remove retired processes which do not have data
         retired_processes.filter(data__exact=None).delete()
@@ -406,7 +392,7 @@ class Command(BaseCommand):
         latest_version_processes = Process.objects.order_by(
             "slug", "-version"
         ).distinct("slug")
-        Process.objects.filter(data__exact=None).exclude(
+        Process.all_objects.filter(data__exact=None).exclude(
             id__in=latest_version_processes
         ).delete()
 
@@ -452,8 +438,6 @@ class Command(BaseCommand):
         self.register_descriptors(
             descriptor_schemas, user_admin, force, verbosity=verbosity
         )
-        # NOTE: Descriptor schemas must be registered first, so
-        #       processes can validate 'entity_descriptor_schema' field.
         self.register_processes(process_schemas, user_admin, force, verbosity=verbosity)
 
         if retire:
