@@ -13,7 +13,9 @@ socket for communication.
 """
 
 import asyncio
+import datetime
 import logging
+import os
 from contextlib import suppress
 from functools import lru_cache
 from time import time
@@ -24,7 +26,6 @@ import zmq.asyncio
 from asgiref.sync import async_to_sync
 from channels.db import database_sync_to_async
 from channels.layers import get_channel_layer
-
 from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.core.exceptions import ValidationError
@@ -34,12 +35,14 @@ from django.utils.timezone import now
 from resolwe.flow.executors.socket_utils import (
     BaseProtocol,
     Message,
+    MessageProcessingEventType,
     PeerIdentity,
     Response,
     ResponseStatus,
 )
 from resolwe.flow.executors.zeromq_utils import ZMQCommunicator
 from resolwe.flow.managers import consumer
+from resolwe.flow.managers.listener.metrics import metrics_reporter
 from resolwe.flow.managers.protocol import WorkerProtocol
 from resolwe.flow.managers.state import LISTENER_CONTROL_CHANNEL  # noqa: F401
 from resolwe.flow.models import Data, Storage, Worker
@@ -48,6 +51,8 @@ from resolwe.storage.models import AccessLog
 from resolwe.test.utils import ignore_in_tests, is_testing
 from resolwe.utils import BraceMessage as __
 
+from .authenticator import ZMQAuthenticator
+
 # Register plugins by importing them.
 from .basic_commands_plugin import BasicCommands  # noqa: F401
 from .bootstrap_plugin import BootstrapCommands  # noqa: F401
@@ -60,6 +65,30 @@ from .redis_cache import RedisLockStatus, cache_manager, redis_server
 logger = logging.getLogger(__name__)
 User = get_user_model()
 
+# Public and private key read from the environment. When the environment does not
+# exist (for instance when running tests), default key is used.
+# The default key can only be used when the DEBUG setting is set to True.
+DEFAULT_LISTENER_PUBLIC_KEY = "*%z%0!H7z84uT^]Gwz89:YQr#@Wm=qB(9Ao+qQ^K"
+DEFAULT_LISTENER_PRIVATE_KEY = ":M-RdvbLHztG+FaaYdp}RKV-fajY}$QT[+XgWD{="
+env_public_key = os.getenv("LISTENER_PUBLIC_KEY", DEFAULT_LISTENER_PUBLIC_KEY)
+env_private_key = os.getenv("LISTENER_PRIVATE_KEY", DEFAULT_LISTENER_PRIVATE_KEY)
+
+if not settings.DEBUG:
+    assert (
+        env_private_key != DEFAULT_LISTENER_PRIVATE_KEY
+        and env_public_key != DEFAULT_LISTENER_PUBLIC_KEY
+    ), "Must set private and public listener key when not running in debug mode"
+
+# Make sure listener private and public keys are encoded.
+LISTENER_PUBLIC_KEY = env_public_key.encode()
+LISTENER_PRIVATE_KEY = env_private_key.encode()
+
+if not settings.DEBUG:
+    assert (
+        LISTENER_PRIVATE_KEY != DEFAULT_LISTENER_PRIVATE_KEY
+        and LISTENER_PUBLIC_KEY != DEFAULT_LISTENER_PUBLIC_KEY
+    ), "Must set private and public listener key when not running in debug mode"
+
 
 class Processor:
     """Process the messages sent by the workers.
@@ -384,7 +413,6 @@ class Processor:
 
         """
         data_id = abs(int(identity))
-
         # Do not proccess messages from Workers that have already finish
         # processing data objects.
         worker_status, data_status, started = self.get_data_fields(
@@ -521,7 +549,13 @@ class ListenerProtocol(BaseProtocol):
         """Initialize."""
         if zmq_socket is None:
             zmq_context: zmq.asyncio.Context = zmq.asyncio.Context.instance()
+            auth = ZMQAuthenticator.instance(zmq_context)
+            auth.start()
             zmq_socket = zmq_context.socket(zmq.ROUTER)
+            # Configure keys and start the server.
+            zmq_socket.curve_secretkey = LISTENER_PRIVATE_KEY
+            zmq_socket.curve_publickey = LISTENER_PUBLIC_KEY
+            zmq_socket.curve_server = True
             zmq_socket.setsockopt(zmq.ROUTER_HANDOVER, 1)
             for host in hosts:
                 zmq_socket.bind(f"{protocol}://{host}:{port}")
@@ -530,6 +564,7 @@ class ListenerProtocol(BaseProtocol):
             ZMQCommunicator(zmq_socket, "listener <-> workers", logger),
             logger,
             max_concurrent_commands,
+            metrics_reporter,
         )
         self.communicator.heartbeat_handler = self.heartbeat_handler
         self._message_processor = Processor(self)
@@ -540,11 +575,16 @@ class ListenerProtocol(BaseProtocol):
         # When the key does not exist in the redis database create one with the
         # current timestamp.
         try:
-            one_day = 24 * 3600
+            one_hour = 3600
+            one_day = 24 * one_hour
             data_id = abs(int(peer_identity))
             redis_key = f"resolwe-worker-{data_id}"
             redis_server.set(redis_key, int(time()))
-            redis_server.expire(redis_key, one_day)
+
+            # Make sure the expiration time is longer than the highest timeout
+            # (currently one week). Otherwise the entry will expire before the timeout
+            # is reached.
+            redis_server.expire(redis_key, 8 * one_day)
         except Exception:
             logger.exception("Exception in heartbeat handler.")
 
@@ -577,11 +617,12 @@ class ListenerProtocol(BaseProtocol):
             )
 
         default_timeout = 600
-        one_day = 24 * 3600
-        one_week = 7 * one_day
+        one_hour = 3600
+        one_day = 24 * one_hour
+
         non_responsive_timeout = {
-            Worker.STATUS_FINISHED_PREPARING: 7200,
-            Worker.STATUS_PREPARING: one_week,
+            Worker.STATUS_FINISHED_PREPARING: 2 * one_hour,
+            Worker.STATUS_PREPARING: 7 * one_day,
         }
 
         current_timestamp = int(time())
@@ -592,26 +633,38 @@ class ListenerProtocol(BaseProtocol):
             last_seen = redis_server.get(redis_key)
             if last_seen is None:
                 redis_server.set(redis_key, current_timestamp)
-                redis_server.expire(redis_key, one_day)
+                # Make sure the expiration time is longer than the highest timeout
+                # (currently one week). Otherwise the entry will expire before the
+                # timeout is reached.
+                redis_server.expire(redis_key, 8 * one_day)
             else:
                 last_seen = int(last_seen)
             without_heartbeat = current_timestamp - (last_seen or current_timestamp)
+            logger.debug(
+                __(
+                    "Worker {} with status {} not seen {} seconds.",
+                    data_id,
+                    worker_status,
+                    without_heartbeat,
+                )
+            )
             if without_heartbeat > non_responsive_timeout.get(
                 worker_status, default_timeout
             ):
                 try:
+                    logger.info(
+                        __(
+                            "Worker {} with status {} marked non-responsive.",
+                            data_id,
+                            worker_status,
+                        )
+                    )
                     await self._message_processor.peer_not_responding(data_id)
                 except Exception:
                     self.logger.exception(
                         "Exception updating unresponsive peer status."
                     )
 
-    async def handle_liveness_probe(
-        self, message: Message, peer_identity: PeerIdentity
-    ) -> Response[bool]:
-        """Respond to the liveness probe."""
-        return message.respond_ok(True)
-
     def _handle_lock_message_error(
         self, lock_status: RedisLockStatus, received_message: Message
     ) -> Response:
@@ -660,6 +713,35 @@ class ListenerProtocol(BaseProtocol):
         """
         response: Optional[Response] = None
         extend_lock_task: Optional[asyncio.Task] = None
+
+        # Handle the liveness probe in a special way (no access control is needed).
+        if received_message.command_name == "liveness_probe":
+            return received_message.respond("OK")
+
+        try:
+            data_id = abs(int(peer_identity))
+        except Exception:
+            error = f"Unable to parse the identity {peer_identity!r}."
+            self.logger.error(error)
+            return received_message.respond_error(error)
+
+        try:
+            # Check that the client is valid and can process the data object.
+            # Before any processing make sure the client is valid and has the permission to
+            # process the data object.
+            key = received_message.client_id
+            authenticator = ZMQAuthenticator.instance()
+            if not authenticator.can_access_data(key, data_id):
+                error_message = (
+                    f"Client with key {key!r} is not allowed to process the data object "
+                    f"with id {data_id}."
+                )
+                self.logger.error(error_message)
+                return received_message.respond_error(error_message)
+        except Exception:
+            error = f"Client {received_message.client_id!r} is not valid."
+            return received_message.respond_error(error)
+
         try:
             # Try to lock the specific message for the data object. If locked it means
             # the message was/is being processed by some other listener.
@@ -669,7 +751,6 @@ class ListenerProtocol(BaseProtocol):
             # message can be processed twice. We can avoid processing the message twice
             # but then all processes that have messages in the listener queue will fail
             # if the listener chashes.
-            data_id = abs(int(peer_identity))
             success, lock_status = cache_manager.lock(
                 Data, [(data_id, received_message.uuid)]
             )[0]
@@ -712,6 +793,26 @@ class ListenerProtocol(BaseProtocol):
         except Exception:
             logger.exception("Error processing post finish command.")
 
+    async def post_init_completed(self, message: Message, peer_identity: PeerIdentity):
+        """Sent metrics about preparation phase duration."""
+        with suppress(Exception):
+            data_id = abs(int(peer_identity))
+            started: datetime.datetime = (
+                await database_sync_to_async(
+                    self._message_processor.get_data_fields,
+                    thread_sensitive=False,
+                )(data_id, ["started"])
+            )[0]
+            try:
+                metrics_reporter.event(
+                    MessageProcessingEventType.PREPARATION_FINISHED,
+                    message,
+                    peer_identity,
+                    started=started.timestamp(),
+                )
+            except:
+                logger.exception("Error processing post init_completed command.")
+
 
 def handle_exceptions(loop, context):
     """Log uncaught exceptions in asyncio."""
