@@ -38,6 +38,9 @@ User = get_user_model()
 # How many times to retry creating new object on slug collision error.
 OBJECT_CREATE_RETRIES = 10
 
+# How many objects to return in one chunk when iterating.
+MAX_CHUNK_SIZE = 10000
+
 PluginType = TypeVar("PluginType")
 
 
@@ -119,6 +122,44 @@ class PythonProcess(ListenerPlugin):
         model_data["contributor_id"] = manager.contributor(data_id).id
         return message.respond_ok(create_model(model, model_data).id)
 
+    def handle_iterate_objects(
+        self,
+        data_id: int,
+        message: Message[Tuple[str, str, dict, list[str], list[str], int]],
+        manager: "Processor",
+    ) -> Response[Dict]:
+        """Get a list of objects based on criteria.
+
+        The list has at most 1000 entries.
+        """
+        # Sorting was added later. For compatibility reasons handle both
+        # message types, remove the one without sorting ASAP.
+
+        sorting: List[str] = []
+        app_name, model_name, filters, sorting, attributes, offset = (
+            message.message_data
+        )
+        full_model_name = f"{app_name}.{model_name}"
+        model = apps.get_model(app_name, model_name)
+        filtered_objects = self._permission_manager.filter_objects(
+            manager.contributor(data_id),
+            full_model_name,
+            model.objects.filter(**filters),
+            data_id,
+        )
+        number_of_objects = filtered_objects.count()
+        to_return = {
+            "number_of_matched_objects": number_of_objects,
+            "chunk_size": MAX_CHUNK_SIZE,
+            "starting_offset": offset,
+            "objects": list(
+                filtered_objects.order_by(*sorting).values_list(*attributes)[
+                    offset : offset + MAX_CHUNK_SIZE
+                ]
+            ),
+        }
+        return message.respond_ok(to_return)
+
     def handle_filter_objects(
         self,
         data_id: int,
@@ -173,13 +214,14 @@ class PythonProcess(ListenerPlugin):
         mapping representing annotations. The keys in the are in the format
         f"{group_name}.{field_name}".
         """
+
         entity_id, annotations, update = message.message_data
         entity = Entity.objects.get(pk=entity_id)
         # Check that the user has the permissions to update the entity.
         self._permission_manager.can_update(
             manager.contributor(data_id), "flow.Entity", entity, {}, data_id
         )
-        entity.update_annotations(annotations, update)
+        entity.update_annotations(annotations, manager.contributor(data_id), update)
         return message.respond_ok("OK")
 
     def handle_get_entity_annotations(
