@@ -14,6 +14,7 @@ import random
 import re
 import string
 import time
+from base64 import b64encode
 from contextlib import suppress
 from enum import Enum
 from importlib.util import find_spec
@@ -22,12 +23,12 @@ from typing import Any, Dict, Iterable, Tuple
 
 import kubernetes
 import redis
-
 from django.conf import settings
 from django.db.models import Sum
 from django.db.models.functions import Coalesce
 
 from resolwe.flow.executors import constants
+from resolwe.flow.managers.listener.listener import LISTENER_PUBLIC_KEY
 from resolwe.flow.models import Data, DataDependency, Process
 from resolwe.flow.utils.decorators import retry
 from resolwe.storage import settings as storage_settings
@@ -95,7 +96,7 @@ def sanitize_kubernetes_label(label: str, trim_end: bool = True) -> str:
     https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/
     """
     max_length = 63
-    sanitized_label = re.sub("[^0-9a-zA-Z\-]+", "-", label).strip("-_.")
+    sanitized_label = re.sub(r"[^0-9a-zA-Z\-]+", "-", label).strip("-_.")
     if len(sanitized_label) > max_length:
         logger.warning(__("Label '%s' is too long and was truncated.", label))
         if trim_end:
@@ -477,6 +478,28 @@ class Connector(BaseConnector):
             "job_type": sanitize_kubernetes_label(job_type),
         }
 
+    def _secrets(self, data: Data, secrets_name: str) -> dict[str, Any]:
+        """Prepare the secrets for the process.
+
+        Every process needs at least:
+        - LISTENER_PUBLIC_KEY
+        - CURVE_PUBLIC_KEY
+        - CURVE_PRIVATE_KEY
+
+        They are necessary in order to communicate with the backend (listener service).
+        """
+        return {
+            "apiVersion": "v1",
+            "kind": "Secret",
+            "metadata": {"name": secrets_name},
+            "immutable": True,
+            "data": {
+                "LISTENER_PUBLIC_KEY": b64encode(LISTENER_PUBLIC_KEY).decode(),
+                "CURVE_PUBLIC_KEY": b64encode(data.worker.public_key).decode(),
+                "CURVE_PRIVATE_KEY": b64encode(data.worker.private_key).decode(),
+            },
+        }
+
     def _persistent_volume_claim(
         self, claim_name: str, size: int, volume_config: Dict, labels: dict[str, str]
     ) -> Dict[str, Any]:
@@ -639,6 +662,7 @@ class Connector(BaseConnector):
             container_name_prefix, data.pk, random_postfix
         )
 
+        annotations = dict()
         # Set resource limits.
         requests = dict()
         limits = data.get_resource_limits()
@@ -682,6 +706,22 @@ class Connector(BaseConnector):
             network = getattr(settings, "FLOW_EXECUTOR", {}).get("NETWORK", "")
             use_host_network = network == "host"
 
+        service_account_name = getattr(
+            settings, "FLOW_KUBERNETES_COMMUNICATOR_SERVICE_ACCOUNT", "default"
+        )
+
+        init_container_name = sanitize_kubernetes_label(f"{container_name}-init")
+        processing_container_name = sanitize_kubernetes_label(f"{container_name}")
+        communicator_container_name = sanitize_kubernetes_label(
+            f"{container_name}-communicator"
+        )
+        # Prevents processing and init containers from assuming IAM role credentials
+        annotations["eks.amazonaws.com/skip-containers"] = ",".join(
+            [
+                processing_container_name,
+            ]
+        )
+
         # Generate and set seccomp policy to limit syscalls.
         processing_uid, processing_gid = self._get_processing_uid_gid()
         security_context = {
@@ -692,8 +732,6 @@ class Connector(BaseConnector):
             "capabilities": {"drop": ["ALL"]},
         }
 
-        annotations = dict()
-
         # Do not evict job from node.
         annotations["cluster-autoscaler.kubernetes.io/safe-to-evict"] = "false"
 
@@ -733,9 +771,18 @@ class Connector(BaseConnector):
             processing_container_image, mapper
         )
 
+        # Create secrets.
+        secrets_name = f"secrets-{data.pk}-{random_postfix}"
+        core_api.create_namespaced_secret(
+            body=self._secrets(data, secrets_name),
+            namespace=self.kubernetes_namespace,
+            _request_timeout=KUBERNETES_TIMEOUT,
+        )
+
         pull_policy = getattr(settings, "FLOW_KUBERNETES_PULL_POLICY", "Always")
         job_type = dict(Process.SCHEDULING_CLASS_CHOICES)[data.process.scheduling_class]
         labels = self._create_labels(data, job_type)
+
         job_description = {
             "apiVersion": "batch/v1",
             "kind": "Job",
@@ -756,6 +803,7 @@ class Connector(BaseConnector):
                     "spec": {
                         "affinity": {},
                         "hostNetwork": use_host_network,
+                        "serviceAccountName": service_account_name,
                         "volumes": self._volumes(
                             data.id,
                             location_subpath,
@@ -765,9 +813,7 @@ class Connector(BaseConnector):
                         ),
                         "initContainers": [
                             {
-                                "name": sanitize_kubernetes_label(
-                                    f"{container_name}-init"
-                                ),
+                                "name": init_container_name,
                                 "image": communicator_image,
                                 "imagePullPolicy": pull_policy,
                                 "workingDir": "/",
@@ -776,11 +822,12 @@ class Connector(BaseConnector):
                                 "securityContext": {"privileged": True},
                                 "volumeMounts": self._init_container_mountpoints(),
                                 "env": container_environment,
+                                "envFrom": [{"secretRef": {"name": secrets_name}}],
                             },
                         ],
                         "containers": [
                             {
-                                "name": sanitize_kubernetes_label(container_name),
+                                "name": processing_container_name,
                                 "image": processing_container_image,
                                 "resources": {"limits": limits, "requests": requests},
                                 "securityContext": security_context,
@@ -794,9 +841,7 @@ class Connector(BaseConnector):
                                 ),
                             },
                             {
-                                "name": sanitize_kubernetes_label(
-                                    f"{container_name}-communicator"
-                                ),
+                                "name": communicator_container_name,
                                 "image": communicator_image,
                                 "imagePullPolicy": pull_policy,
                                 "resources": {
@@ -805,6 +850,7 @@ class Connector(BaseConnector):
                                 },
                                 "securityContext": security_context,
                                 "env": container_environment,
+                                "envFrom": [{"secretRef": {"name": secrets_name}}],
                                 "command": ["/usr/local/bin/python3"],
                                 "args": ["/startup.py"],
                                 "volumeMounts": self._communicator_mountpoints(
@@ -873,6 +919,41 @@ class Connector(BaseConnector):
             "It took {:.2f}s to send config to kubernetes".format(end_time - start_time)
         )
 
+        # Patch the secrets with owner references to the job.
+        patch = [
+            {
+                "op": "add",
+                "path": "/metadata/ownerReferences",
+                "value": [
+                    {
+                        "apiVersion": "batch/v1",
+                        "kind": "Job",
+                        "name": job.metadata.name,
+                        "uid": job.metadata.uid,
+                    }
+                ],
+            }
+        ]
+
+        # Patch the secret with ownerReference. This ensures the secret gets deleted
+        # when the job has completed or failed. The secrets are deleted after
+        # ttlSecondsAfterFinished seconds, declared in the Job spec.
+        try:
+            core_api.patch_namespaced_secret(
+                name=secrets_name, namespace=self.kubernetes_namespace, body=patch
+            )
+        except Exception as error:
+            # This operation is not critical for the job to run, so we log the
+            # error and continue. The job will still run, but the PVC will not be
+            # deleted automatically after the job has finished.
+            logger.exception(
+                __(
+                    "Kubernetes owner patch for secret {} failed: '{}'.",
+                    secrets_name,
+                    error,
+                )
+            )
+
         # Patch the PVC-s with owner references to the job.
         patch = [
             {
@@ -891,7 +972,7 @@ class Connector(BaseConnector):
         for claim_name in created_claim_names:
             # Patch the PVC with ownerReference. This ensures the PVC claim gets deleted
             # when the job has completed or failed. The PVCs are deleted after
-            # ttlSecondsAfterFinished seconds, declared in Job spec.
+            # ttlSecondsAfterFinished seconds, declared in the Job spec.
             try:
                 core_api.patch_namespaced_persistent_volume_claim(
                     name=claim_name, namespace=self.kubernetes_namespace, body=patch
