@@ -5,6 +5,8 @@ import datetime
 import re
 from collections import defaultdict
 from enum import Enum
+from functools import reduce
+from itertools import batched
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -15,18 +17,29 @@ from typing import (
     Optional,
     Sequence,
     Type,
+    TypedDict,
 )
 
 from django.core.exceptions import ValidationError
-from django.db import models
+from django.db import models, transaction
 
-from resolwe.flow.models.base import BaseModel
-from resolwe.permissions.models import Permission, PermissionObject
+from resolwe.flow.models.base import (
+    MAX_SLUG_LENGTH,
+    BaseManager,
+    BaseModel,
+    ResolweSlugField,
+)
+from resolwe.flow.models.fields import VersionField
+from resolwe.permissions.models import (
+    PermissionInterface,
+    PermissionObject,
+    PermissionQuerySet,
+)
 
 if TYPE_CHECKING:
     from resolwe.flow.models import Collection, Entity
 
-from .base import AuditModel
+from .base import VERSION_NUMBER_BITS, AuditModel
 
 VALIDATOR_LENGTH = 128
 NAME_LENGTH = 128
@@ -212,8 +225,8 @@ class AnnotationFieldVocabularyValidator(AnnotationFieldBaseValidator):
             )
 
 
-class AnnotationGroup(models.Model):
-    """Group of annotation fields."""
+class AbstractAnnotationGroup(models.Model):
+    """Abstract base class for annotation groups."""
 
     #: the name of the annotation group
     name = models.CharField(max_length=NAME_LENGTH)
@@ -224,18 +237,41 @@ class AnnotationGroup(models.Model):
     #: the sorting order among annotation groups
     sort_order = models.PositiveSmallIntegerField()
 
-    def __str__(self) -> str:
-        """Return user-friendly string representation."""
-        return f"{self.name}"
+    class Meta:
+        """Make this class abstract."""
+
+        abstract = True
+
+
+class AnnotationGroup(AbstractAnnotationGroup):
+    """Group of annotation fields."""
 
     class Meta:
         """Set the default ordering."""
 
         ordering = ["sort_order"]
 
+    def __str__(self) -> str:
+        """Return user-friendly string representation."""
+        return f"{self.name}"
 
-class AnnotationField(models.Model):
-    """Annotation field."""
+
+class AnnotationFieldManager(
+    BaseManager["AnnotationField", models.QuerySet["AnnotationField"]]
+):
+    """Annotation field manager.
+
+    Return only the latest version of each field.
+    """
+
+    partition_fields = ["name", "group"]
+
+
+class AbstractAnnotationField(models.Model):
+    """Abstract base class for annotation fields."""
+
+    partition_fields = ["name", "group_id"]
+    version_field = "version"
 
     #: the name of the annotation fields
     name = models.CharField(max_length=NAME_LENGTH)
@@ -268,6 +304,14 @@ class AnnotationField(models.Model):
     #: is this field required
     required = models.BooleanField(default=False)
 
+    #: field version
+    version = VersionField(number_bits=VERSION_NUMBER_BITS, default="0.0.0")
+
+    class Meta:
+        """Make this class abstract."""
+
+        abstract = True
+
     def __init__(self, *args, **kwargs):
         """Store original vocabulary to private variable."""
         super().__init__(*args, **kwargs)
@@ -294,6 +338,11 @@ class AnnotationField(models.Model):
             raise ValidationError(f"Invalid path '{path}'.")
         return [match["group"], match["field"]]
 
+    @property
+    def path(self) -> str:
+        """Return the path of the field."""
+        return f"{self.group.name}.{self.name}"
+
     @classmethod
     def id_from_path(cls, path: str) -> Optional[int]:
         """Get the field id from the field path."""
@@ -329,7 +378,7 @@ class AnnotationField(models.Model):
                 annotation_value_field.recompute_label()
                 annotation_value_field.validate()
                 updated_fields.append(annotation_value_field)
-            AnnotationValue.objects.bulk_update(updated_fields, ["_value"])
+            AnnotationValue.all_objects.bulk_update(updated_fields, ["_value"])
             self._original_vocabulary = self.vocabulary
         if self.type != self._original_type:
             self.revalidate_values()
@@ -354,9 +403,15 @@ class AnnotationField(models.Model):
                     return vocabulary_value
         return label
 
-    def __str__(self) -> str:
-        """Return user-friendly string representation."""
-        return f"{self.group.name}.{self.name}"
+
+class AnnotationField(AbstractAnnotationField):
+    """Annotation field."""
+
+    #: the latest version of the models
+    objects = AnnotationFieldManager()
+
+    #: all models
+    all_objects = models.Manager()
 
     class Meta:
         """Set the constraints and the default ordering."""
@@ -368,13 +423,18 @@ class AnnotationField(models.Model):
                 name="annotation_type",
             ),
             models.constraints.UniqueConstraint(
-                fields=["name", "group"], name="uniquetogether_name_group"
+                fields=["name", "group", "version"],
+                name="uniquetogether_name_group_version",
             ),
         ]
         ordering = ["group__sort_order", "sort_order"]
 
+    def __str__(self) -> str:
+        """Return user-friendly string representation."""
+        return f"{self.group.name}.{self.name}"
 
-class AnnotationPreset(BaseModel, PermissionObject):
+
+class AnnotationPreset(PermissionObject, BaseModel):
     """The named set of annotation fields.
 
     The presets have permissions.
@@ -383,62 +443,147 @@ class AnnotationPreset(BaseModel, PermissionObject):
     #: the fields belonging to this preset
     fields = models.ManyToManyField(AnnotationField, related_name="presets")
 
-    class Meta:
-        """Override parent meta."""
-
     def __str__(self):
         """Return the string representation."""
         return self.name
 
-
-class AnnotationValueQuerySet(models.QuerySet):
-    """Custom queryset for AnnotationValue."""
-
-    def filter_for_user(self, user) -> "AnnotationValueQuerySet":
-        """Filter annotation values for user.
-
-        Return the annotation values on entities user has view permission for.
-        """
-        # Avoid circular import.
-        from resolwe.flow.models import Entity
-
-        return self.filter(
-            entity__in=Entity.objects.filter(
-                pk__in=self.values("entity")
-            ).filter_for_user(user)
-        )
-
-
-class AnnotationValue(AuditModel):
-    """The value of the annotation."""
+    def can_set_permission(self) -> bool:
+        """Return whether permissions can be set."""
+        return True
 
     class Meta:
-        """Set the unique constraints."""
+        """Override parent meta."""
 
-        constraints = [
-            models.constraints.UniqueConstraint(
-                fields=["entity", "field"], name="uniquetogether_entity_field"
-            ),
-        ]
-        ordering = ["field__group__sort_order", "field__sort_order"]
 
-    objects = AnnotationValueQuerySet.as_manager()
+class AddAnnotationDict(TypedDict):
+    """The type of the dictionary for bulk adding annotations."""
+
+    field: AnnotationField
+    entity: "Entity"
+    contributor: Any
+    value: Any
+
+
+class AnnotationValueManager(BaseManager["AnnotationValue", PermissionQuerySet]):
+    """Annotation value manager.
+
+    Return only the latest version of each field.
+    """
+
+    QuerySet = PermissionQuerySet
+    enable_versioning = False
+
+    def get_queryset(self):
+        """Return only the values with delete marker not set."""
+        return super().get_queryset().filter(deleted=False)
+
+    def add_annotations(
+        self, annotations: Iterable[AddAnnotationDict]
+    ) -> list["AnnotationValue"]:
+        """Bulk add annotations.
+
+        We have to consider the possibilities:
+        - the value is not a delete marker:
+          - the existing value exists:
+            - create a new entry and set deleted to true on the existing one if the value
+              has actually changed.
+          - the value does not exist yet or is deleted: create a new entry.
+        - the value is a delete marker:
+          - the existing value exists: set deleted to true.
+        """
+
+        def is_delete_marker(value):
+            """Is the value a delete marker."""
+            return value is None
+
+        def combine_query(query: models.Q, data: dict) -> models.Q:
+            """Combine the query with the data."""
+            return query | models.Q(field=data["field"], entity=data["entity"])
+
+        # Make method more general for use in classed derived from AnnotationValue.
+        Model = self.model
+        # The annotations are processed in batches to avoid creating too complex
+        # queries. The BATCH_SIZE determines the size of the batch.
+        BATCH_SIZE = 1000
+        created = []
+        for batch in batched(annotations, BATCH_SIZE):
+            to_process = {(data["field"].pk, data["entity"].pk): data for data in batch}
+            # Create a mapping to easy access the existing values.
+            existing_values = {
+                (value.field_id, value.entity_id): value
+                for value in Model.objects.filter(
+                    reduce(combine_query, batch, models.Q())
+                )
+            }
+            # We have to:
+            # - create new values
+            # - create new values and delete existing ones if the value has changed
+            # - delete values with None
+            to_delete = []
+            to_create = []
+            for key, data in to_process.items():
+                if existing_value := existing_values.get(key):
+                    if is_delete_marker(data["value"]):
+                        to_delete.append(existing_value.pk)
+                    elif existing_value.value != data["value"]:
+                        to_create.append(data)
+                        to_delete.append(existing_value.pk)
+                elif not is_delete_marker(data["value"]):
+                    to_create.append(data)
+
+            # Create new objects we must call the save method to set the delete marker.
+            created_values = [Model(**data) for data in to_create]
+            for value in created_values:
+                value.save()
+
+            created.extend(created_values)
+            # Mark objects as deleted.
+            if to_delete:
+                Model.objects.filter(pk__in=to_delete).update(deleted=True)
+
+        # Only return created objects.
+        return created
+
+
+def _slug_for_annotation_value(instance: "AnnotationValue") -> str:
+    """Generate the slug for the annotation value."""
+    return f"{instance.entity.slug}-{instance.field.group.name}-{instance.field.name}"
+
+
+class AbstractAnnotationValue(BaseModel):
+    """Abstract base class for annotation values."""
+
+    partition_fields = ["field", "entity"]
+    version_field = "created"
+
+    #: URL slug
+    slug = ResolweSlugField(
+        populate_from=_slug_for_annotation_value,
+        unique_with="version",
+        max_length=MAX_SLUG_LENGTH,
+    )
 
     #: the entity this field belongs to
     entity: "Entity" = models.ForeignKey(
         "Entity", related_name="annotations", on_delete=models.CASCADE
-    )
+    )  # type: ignore
 
     #: the field this field belongs to
-    field: AnnotationField = models.ForeignKey(
+    field = models.ForeignKey(
         AnnotationField, related_name="values", on_delete=models.PROTECT
     )
 
-    #: the date when field was last modified
-    modified = models.DateTimeField(auto_now=True)
+    #: the delete marker
+    deleted = models.BooleanField(default=False)
 
     #: value is stored under key 'value' in the json field to simplify lookups
-    _value: Any = models.JSONField(default=dict)
+    #: the value None is used as delete marker
+    _value: Any = models.JSONField(default=dict, null=True)
+
+    class Meta:
+        """Make this class abstract."""
+
+        abstract = True
 
     def __init__(self, *args, **kwargs):
         """Allow us to set the 'value' in the constructor.
@@ -453,9 +598,14 @@ class AnnotationValue(AuditModel):
         if "label" not in self._value:
             self.recompute_label()
 
+    @property
+    def delete_marker(self) -> bool:
+        """Return True if this value represents a delete marker."""
+        return self.deleted
+
     @property
     def value(self) -> str | int | float | datetime.date:
-        """Get the actual value."""
+        """Get the actual value or None if object is delete marker."""
         return self._value["value"]
 
     @value.setter
@@ -493,14 +643,23 @@ class AnnotationValue(AuditModel):
 
         :raises ValidationError: when the validation fails.
         """
-        annotation_value_validator.validate(self, raise_exception=True)
+        if not self.delete_marker:
+            annotation_value_validator.validate(self, raise_exception=True)
 
+    @transaction.atomic
     def save(self, *args, **kwargs):
         """Save the annotation value after validation has passed."""
-        annotation_value_validator.validate(self, raise_exception=True)
-        # Make sure the label is always set.
-        if "label" not in self._value:
-            self.recompute_label()
+        # Set deleted on other instances with the same field and entity.
+        if self._state.adding:
+            type(self).all_objects.filter(
+                field=self.field, entity=self.entity, deleted=False
+            ).update(deleted=True)
+
+        if not self.delete_marker:
+            annotation_value_validator.validate(self, raise_exception=True)
+            # Make sure the label is always set.
+            if "label" not in self._value:
+                self.recompute_label()
         super().save(*args, **kwargs)
 
     def recompute_label(self):
@@ -515,13 +674,39 @@ class AnnotationValue(AuditModel):
             entity_id=entity_id, field_id=field_id
         ).first()
 
-    def has_permission(self, permission: Permission, user) -> bool:
-        """Return if user permission on this object.
+    @classmethod
+    def permission_proxy(cls) -> str:
+        """Return the permission group path."""
+        return "entity"
 
-        The permission is checked on the entity.
-        """
-        return self.entity.has_permission(permission, user)
+    def in_container(self) -> bool:
+        """Return if object lies in a container."""
+        return True
+
+
+class AnnotationValue(AbstractAnnotationValue, PermissionInterface, AuditModel):
+    """The value of the annotation."""
+
+    #: the latest version of the models
+    objects = AnnotationValueManager()  # type: ignore
+
+    #: all models
+    all_objects: models.Manager["AnnotationValue"] = PermissionQuerySet.as_manager()
+
+    class Meta:
+        """Set the unique constraints."""
+
+        constraints = [
+            models.constraints.UniqueConstraint(
+                fields=["entity", "field", "created"],
+                name="uniquetogether_entity_field_created",
+            ),
+        ]
+        ordering = ["field__group__sort_order", "field__sort_order"]
 
     def __str__(self) -> str:
         """Return user-friendly string representation."""
-        return f"{self.label}"
+        if self.delete_marker:
+            return "Delete marker"
+        else:
+            return f"{self.label}"
