@@ -7,7 +7,6 @@ import logging
 from typing import Any, Optional, Union
 
 import jsonschema
-
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.postgres.fields import ArrayField
 from django.contrib.postgres.indexes import GinIndex
@@ -17,6 +16,7 @@ from django.core.validators import RegexValidator
 from django.db import models, transaction
 
 from resolwe.flow.expression_engines.exceptions import EvaluationError
+from resolwe.flow.models.base import BaseQuerySet
 from resolwe.flow.models.utils import (
     DirtyError,
     fill_with_defaults,
@@ -28,11 +28,10 @@ from resolwe.observers.consumers import BackgroundTaskType
 from resolwe.observers.decorators import move_to_container
 from resolwe.observers.models import BackgroundTask
 from resolwe.observers.utils import start_background_task
-from resolwe.permissions.models import PermissionObject, PermissionQuerySet
+from resolwe.permissions.models import PermissionObject
 from resolwe.permissions.utils import assign_contributor_permissions, copy_permissions
 
-from .base import BaseModel, BaseQuerySet
-from .descriptor import DescriptorSchema
+from .base import BaseModel, DataStatus
 from .entity import Entity, EntityQuerySet
 from .history_manager import HistoryMixin
 from .secret import Secret
@@ -61,7 +60,7 @@ class HandleEntityOperation(enum.Enum):
     PASS = "PASS"
 
 
-class DataQuerySet(BaseQuerySet, PermissionQuerySet):
+class DataQuerySet(BaseQuerySet):
     """Query set for Data objects."""
 
     @staticmethod
@@ -79,7 +78,6 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
 
         """
         entity_type = obj.process.entity_type
-        entity_descriptor_schema = obj.process.entity_descriptor_schema
         entity_input = obj.process.entity_input
         entity_always_create = obj.process.entity_always_create
         operation = HandleEntityOperation.PASS
@@ -110,12 +108,8 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
             entity_count = entity_query.count()
 
             if entity_count == 0 or entity_always_create:
-                descriptor_schema = DescriptorSchema.objects.filter(
-                    slug=entity_descriptor_schema
-                ).latest()
                 entity = Entity.objects.create(
                     contributor=obj.contributor,
-                    descriptor_schema=descriptor_schema,
                     type=entity_type,
                     name=obj.name,
                     tags=obj.tags,
@@ -202,7 +196,7 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
 
         return obj
 
-    def duplicate(self, contributor) -> BackgroundTask:
+    def duplicate(self, request_user) -> BackgroundTask:
         """Duplicate (make a copy) ``Data`` objects in the background."""
         task_data = {
             "data_ids": list(self.values_list("pk", flat=True)),
@@ -210,28 +204,34 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
             "inherit_collection": True,
         }
         return start_background_task(
-            BackgroundTaskType.DUPLICATE_DATA, "Duplicate data", task_data, contributor
+            BackgroundTaskType.DUPLICATE_DATA, "Duplicate data", task_data, request_user
         )
 
-    def delete_background(self, contributor):
+    def delete_background(self, request_user):
         """Delete the ``Data`` objects in the background."""
         task_data = {
             "object_ids": list(self.values_list("pk", flat=True)),
             "content_type_id": ContentType.objects.get_for_model(self.model).pk,
         }
         return start_background_task(
-            BackgroundTaskType.DELETE, "Delete data", task_data, contributor
+            BackgroundTaskType.DELETE, "Delete data", task_data, request_user
         )
 
-    @transaction.atomic
-    def move_to_collection(self, destination_collection):
-        """Move data objects to destination collection.
+    def move_to_collection(self, destination_collection, request_user):
+        """Move data objects to destination collection in the background.
 
         Note that this method will also copy tags and permissions
         of the destination collection to the data objects.
         """
-        for data in self:
-            data.move_to_collection(destination_collection)
+        task_data = {
+            "target_id": destination_collection.pk,
+            "data_ids": list(self.values_list("pk", flat=True)),
+            "entity_ids": [],
+            "request_user_id": request_user.pk,
+        }
+        return start_background_task(
+            BackgroundTaskType.MOVE, "Delete data", task_data, request_user
+        )
 
     def annotate_sample_path(self, path, annotation_name, value_to_label=False):
         """Add annotation to the Entity QuerySet.
@@ -255,7 +255,7 @@ class DataQuerySet(BaseQuerySet, PermissionQuerySet):
         return self.annotate(**annotation_data)
 
 
-class Data(HistoryMixin, BaseModel, PermissionObject):
+class Data(HistoryMixin, PermissionObject, BaseModel):
     """Postgres model for storing data."""
 
     class Meta(BaseModel.Meta):
@@ -277,24 +277,33 @@ class Data(HistoryMixin, BaseModel, PermissionObject):
             models.Index(name="idx_data_status", fields=["status"]),
             GinIndex(name="idx_data_tags", fields=["tags"]),
             GinIndex(name="idx_data_search", fields=["search"]),
+            models.Index(
+                models.Case(
+                    *[
+                        models.When(status=status, then=models.Value(sort_order))
+                        for sort_order, status in enumerate(DataStatus.sort_order())
+                    ]
+                ),
+                name="idx_data_status_priority",
+            ),
         ]
 
     #: data object is uploading
-    STATUS_UPLOADING = "UP"
+    STATUS_UPLOADING = DataStatus.UPLOADING
     #: data object is being resolved
-    STATUS_RESOLVING = "RE"
+    STATUS_RESOLVING = DataStatus.RESOLVING
     #: data object is waiting
-    STATUS_WAITING = "WT"
+    STATUS_WAITING = DataStatus.WAITING
     #: data object is preparing
-    STATUS_PREPARING = "PP"
+    STATUS_PREPARING = DataStatus.PREPARING
     #: data object is processing
-    STATUS_PROCESSING = "PR"
+    STATUS_PROCESSING = DataStatus.PROCESSING
     #: data object is done
-    STATUS_DONE = "OK"
+    STATUS_DONE = DataStatus.DONE
     #: data object is in error state
-    STATUS_ERROR = "ER"
+    STATUS_ERROR = DataStatus.ERROR
     #: data object is in dirty state
-    STATUS_DIRTY = "DR"
+    STATUS_DIRTY = DataStatus.DIRTY
     # Assumption (in listener): ordered from least to most problematic.
     STATUS_CHOICES = (
         (STATUS_UPLOADING, "Uploading"),
@@ -599,7 +608,7 @@ class Data(HistoryMixin, BaseModel, PermissionObject):
         """Return True if data object is a duplicate."""
         return bool(self.duplicated)
 
-    def duplicate(self, contributor) -> BackgroundTask:
+    def duplicate(self, request_user) -> BackgroundTask:
         """Duplicate (make a copy) object in the background."""
         task_data = {
             "data_ids": [self.pk],
@@ -607,7 +616,7 @@ class Data(HistoryMixin, BaseModel, PermissionObject):
             "inherit_collection": True,
         }
         return start_background_task(
-            BackgroundTaskType.DUPLICATE_DATA, "Duplicate data", task_data, contributor
+            BackgroundTaskType.DUPLICATE_DATA, "Duplicate data", task_data, request_user
         )
 
     @move_to_container
@@ -634,6 +643,33 @@ class Data(HistoryMixin, BaseModel, PermissionObject):
                 self.tags = entity.tags
         self.save(update_fields=["permission_group", "entity", "tags"])
 
+    @classmethod
+    def validate_change_containers(cls, data, entity, collection):
+        """Validate changing entity and collection.
+
+        Validate that:
+        - data is not removed from the container.
+        - if entity is given its collection must be the same as given collection
+        """
+        if data and data.in_container():
+            if (data.entity and not entity) or (data.collection and not collection):
+                raise ValidationError(
+                    "Data object can not be removed from the container."
+                )
+        if entity and entity.collection != collection:
+            raise ValidationError(
+                "Entity must belong to the same collection as data object."
+            )
+
+    def move_to_containers(self, entity, collection):
+        """Move the data object to the given entity and collection."""
+        Data.validate_change_containers(self, entity, collection)
+        if self.entity != entity or self.collection != collection:
+            self.collection = collection
+            self.entity = entity
+            self.tags = getattr(collection, "tags", []) or getattr(entity, "tags", [])
+            self.save(update_fields=["collection", "entity", "tags"])
+
     def validate_change_collection(self, collection):
         """Raise validation error if data object cannot change collection."""
         if self.entity and self.entity.collection != collection:
@@ -644,6 +680,17 @@ class Data(HistoryMixin, BaseModel, PermissionObject):
         if collection is None:
             raise ValidationError("Data object can not be removed from the container.")
 
+    def validate_change_entity(self, entity):
+        """Raise validation error if data object cannot change entity."""
+        if entity is None:
+            raise ValidationError("Data object can not be removed from the container.")
+
+        if self.collection and self.collection != entity.collection:
+            raise ValidationError(
+                "If data is in collection, you can only move it to another entity "
+                "in the same collection."
+            )
+
     def _render_name(self):
         """Render data name.
 
