@@ -1,5 +1,6 @@
 """Resolwe entity model."""
 
+from itertools import batched
 from typing import Any, List, Optional, Union
 
 from django.contrib.contenttypes.models import ContentType
@@ -8,21 +9,22 @@ from django.core.exceptions import ValidationError
 from django.db import models, transaction
 
 from resolwe.flow.models.annotations import AnnotationField, AnnotationValue
+from resolwe.flow.models.base import BaseQuerySet
 from resolwe.observers.consumers import BackgroundTaskType
 from resolwe.observers.decorators import move_to_container
 from resolwe.observers.models import BackgroundTask
 from resolwe.observers.utils import start_background_task
-from resolwe.permissions.models import PermissionObject, PermissionQuerySet
+from resolwe.permissions.models import PermissionObject
 
-from .base import BaseModel, BaseQuerySet
-from .collection import BaseCollection, Collection
+from .base import BaseModel
+from .collection import BaseCollection, BaseCollectionQuerySet, Collection
 from .utils import DirtyError, validate_schema
 
 
-class EntityQuerySet(BaseQuerySet, PermissionQuerySet):
+class EntityQuerySet(BaseCollectionQuerySet):
     """Query set for ``Entity`` objects."""
 
-    def duplicate(self, contributor) -> BackgroundTask:
+    def duplicate(self, request_user) -> BackgroundTask:
         """Duplicate (make a copy) ``Entity`` objects in the background."""
         task_data = {
             "entity_ids": list(self.values_list("pk", flat=True)),
@@ -32,24 +34,30 @@ class EntityQuerySet(BaseQuerySet, PermissionQuerySet):
             BackgroundTaskType.DUPLICATE_ENTITY,
             "Duplicate entity",
             task_data,
-            contributor,
+            request_user,
         )
 
-    def delete_background(self, contributor):
+    def delete_background(self, request_user):
         """Delete the ``Entity`` objects in the background."""
         task_data = {
             "object_ids": list(self.values_list("pk", flat=True)),
             "content_type_id": ContentType.objects.get_for_model(self.model).pk,
         }
         return start_background_task(
-            BackgroundTaskType.DELETE, "Delete entities", task_data, contributor
+            BackgroundTaskType.DELETE, "Delete entities", task_data, request_user
         )
 
-    @transaction.atomic
-    def move_to_collection(self, destination_collection: Collection):
+    def move_to_collection(self, destination_collection: Collection, request_user):
         """Move entities to destination collection."""
-        for entity in self:
-            entity.move_to_collection(destination_collection)
+        task_data = {
+            "target_id": destination_collection.pk,
+            "data_ids": [],
+            "entity_ids": list(self.values_list("pk", flat=True)),
+            "request_user_id": request_user.pk,
+        }
+        return start_background_task(
+            BackgroundTaskType.MOVE, "Move entities", task_data, request_user
+        )
 
     def annotate_all(self, add_labels: bool = False):
         """Annotate with all metadata on the entities.
@@ -157,7 +165,7 @@ class EntityQuerySet(BaseQuerySet, PermissionQuerySet):
         return queryset
 
 
-class Entity(BaseCollection, PermissionObject):
+class Entity(PermissionObject, BaseCollection):
     """Postgres model for storing entities."""
 
     class Meta(BaseCollection.Meta):
@@ -208,15 +216,18 @@ class Entity(BaseCollection, PermissionObject):
         ).first()
         return annotation.value if annotation else default
 
-    def set_annotation(self, path: str, value: Any):
+    def set_annotation(self, path: str, value: Any, contributor=None):
         """Get the annotation for the given path.
 
         :attr path: the path to the annotation in the format 'group.field'.
         :attr value: the annotation value.
         """
         field_id = AnnotationField.id_from_path(path)
+        contributor = contributor or self.contributor
         if field_id is not None:
-            AnnotationValue.objects.create(field_id=field_id, entity=self, value=value)
+            AnnotationValue.objects.create(
+                field_id=field_id, entity=self, value=value, contributor=contributor
+            )
         else:
             raise RuntimeError(f"No annotation field for path {path}.")
 
@@ -224,14 +235,14 @@ class Entity(BaseCollection, PermissionObject):
         """Return True if entity is a duplicate."""
         return bool(self.duplicated)
 
-    def duplicate(self, contributor) -> BackgroundTask:
+    def duplicate(self, request_user) -> BackgroundTask:
         """Duplicate (make a copy) object in the background."""
         task_data = {"entity_ids": [self.pk], "inherit_collection": True}
         return start_background_task(
             BackgroundTaskType.DUPLICATE_ENTITY,
             "Duplicate entity",
             task_data,
-            contributor,
+            request_user,
         )
 
     def delete_background(self):
@@ -272,7 +283,8 @@ class Entity(BaseCollection, PermissionObject):
         self.tags = collection.tags
         self.permission_group = collection.permission_group
         self.save(update_fields=["collection", "tags", "permission_group"])
-        self.data.move_to_collection(collection)
+        for data in self.data.all():
+            data.move_to_collection(collection)
 
     def invalid_annotation_fields(self, annotation_fields=None):
         """Get the Queryset of invalid annotation fields.
@@ -289,7 +301,9 @@ class Entity(BaseCollection, PermissionObject):
             collection=self.collection
         )
 
-    def copy_annotations(self, destination: "Entity") -> List[AnnotationValue]:
+    def copy_annotations(
+        self, destination: "Entity", contributor
+    ) -> List[AnnotationValue]:
         """Copy annotation from this entity to the destination.
 
         The objects are not saved to the database.
@@ -311,6 +325,7 @@ class Entity(BaseCollection, PermissionObject):
                 entity=destination,
                 field_id=source_annotation.field_id,
                 value=source_annotation.value,
+                contributor=contributor or source_annotation.contributor,
             )
             destination_annotations.append(annotation_value)
             annotation_field_ids.add(source_annotation.field_id)
@@ -342,56 +357,44 @@ class Entity(BaseCollection, PermissionObject):
         if validation_errors:
             raise ValidationError(validation_errors)
 
-    def update_annotations(self, annotations: dict[str, Any], update=True):
+    @transaction.atomic
+    def update_annotations(
+        self, annotations: dict[str, Any], contributor, update: bool = True
+    ):
         """Update annotations with the given values.
 
-        When annotation value is set no None it is deleted.
+        When annotation value is set no None the existing value is deleted.
 
         :attr annotations: the dictionary with annotation values. Keys are annotation
             paths.
         """
-        field_paths = annotations.keys()
+        # The annotations are processed in batches to avoid creating too complex
+        # queries. The BATCH_SIZE determines the size of the batch.
+        BATCH_SIZE = 1000
         field_map = {
             field_path: AnnotationField.field_from_path(field_path)
-            for field_path in field_paths
+            for field_path in set(annotations.keys())
         }
-
-        # Create annotation values and prepare list of fields which annotations should
-        # be deleted.
-        annotation_values: list[AnnotationValue] = []
-        validation_errors: list[ValidationError] = []
-        fields_to_delete: list[int] = []
-        for field_path, field_value in annotations.items():
-            if field_value is None:
-                fields_to_delete.append(field_map[field_path].pk)
-            else:
-                value = AnnotationValue(
-                    field=field_map[field_path], value=field_value, entity=self
-                )
-                annotation_values.append(value)
-                try:
-                    value.validate()
-                except ValidationError as e:
-                    validation_errors.append(e)
-        if validation_errors:
-            raise ValidationError(validation_errors)
-
-        # Delete and update annotations in a transaction.
-        with transaction.atomic():
-            to_delete = AnnotationValue.objects.filter(entity=self)
-            if update:
-                to_delete = to_delete.filter(field_id__in=fields_to_delete)
-            to_delete.delete()
-            AnnotationValue.objects.bulk_create(
-                annotation_values,
-                update_conflicts=True,
-                update_fields=["_value"],
-                unique_fields=["entity", "field"],
-            )
-
-        # Add missing annotation fields to the collection.
-        if self.collection is not None:
-            self.collection.annotation_fields.add(*field_map.values())
+        annotations = AnnotationValue.objects.add_annotations(
+            [
+                {
+                    "field": field_map[field_path],
+                    "entity": self,
+                    "contributor": contributor,
+                    "value": value,
+                }
+                for field_path, value in annotations.items()
+            ]
+        )
+        # Old existing annotations must be deleted.
+        if not update:
+            created_entity_value_ids = {
+                annotation.pk for annotation in annotations if not annotation.deleted
+            }
+            for pk_batch in batched(created_entity_value_ids, BATCH_SIZE):
+                AnnotationValue.objects.filter(entity=self).exclude(
+                    pk__in=created_entity_value_ids
+                ).update(deleted=True)
 
 
 class RelationType(models.Model):
@@ -490,7 +493,7 @@ class Relation(BaseModel, PermissionObject):
     descriptor_dirty = models.BooleanField(default=False)
 
     #: custom manager with permission filtering methods
-    objects = PermissionQuerySet.as_manager()
+    objects = BaseQuerySet.as_manager()
 
     class Meta(BaseModel.Meta):
         """Relation Meta options."""
