@@ -6,7 +6,6 @@ from django.core.exceptions import ValidationError
 from django.db import DEFAULT_DB_ALIAS, connections
 from django.test.utils import CaptureQueriesContext
 from django.urls import reverse
-
 from rest_framework import status
 from rest_framework.test import APIClient, APIRequestFactory, force_authenticate
 
@@ -53,7 +52,6 @@ class TestDuplicate(TransactionTestCase):
             slug="test-process",
             version="1.0.0",
             contributor=self.contributor,
-            entity_descriptor_schema="test-schema",
             input_schema=[
                 {"name": "input_data", "type": "data:test:", "required": False}
             ],
@@ -92,10 +90,7 @@ class TestDuplicate(TransactionTestCase):
         )
 
         self.entity = Entity.objects.create(
-            collection=self.collection,
-            contributor=self.contributor,
-            descriptor_schema=self.descriptor_schema,
-            name="Test entity",
+            collection=self.collection, contributor=self.contributor, name="Test entity"
         )
         self.data = Data.objects.create(
             name="Test data",
@@ -257,6 +252,7 @@ class TestDuplicate(TransactionTestCase):
         force_authenticate(request, self.contributor)
         response = self.entity_duplicate_viewset(request)
         task = BackgroundTask.objects.get(pk=response.data["id"])
+        task.result()
         duplicate = Entity.objects.get(id__in=task.result())
 
         self.assertTrue(task.has_permission(Permission.VIEW, self.contributor))
@@ -283,7 +279,10 @@ class TestDuplicate(TransactionTestCase):
             type="STRING",
         )
         AnnotationValue.objects.create(
-            field=annotation_field1, entity=entity, value="test"
+            field=annotation_field1,
+            entity=entity,
+            value="test",
+            contributor=self.contributor,
         )
         entity.collection.annotation_fields.add(annotation_field1)
         request = factory.post(
@@ -294,7 +293,7 @@ class TestDuplicate(TransactionTestCase):
         force_authenticate(request, self.contributor)
         response = self.entity_duplicate_viewset(request)
         task = BackgroundTask.objects.get(pk=response.data["id"])
-        result = task.result(final_statuses=["OK"])
+        result = task.result()
         duplicate = Entity.objects.get(id__in=result)
         self.assertTrue(task.has_permission(Permission.VIEW, self.contributor))
         self.assertEqual(len(result), 1)
@@ -338,7 +337,7 @@ class TestDuplicate(TransactionTestCase):
         self.assertEqual(collection_without_perm.data.count(), 1)
 
 
-class TestDataViewSetCase(TestCase):
+class TestDataViewSetCaseMixin:
     def setUp(self):
         super().setUp()
         self.client = APIClient()
@@ -380,7 +379,6 @@ class TestDataViewSetCase(TestCase):
             version="1.0.0",
             contributor=self.contributor,
             entity_type="test-schema",
-            entity_descriptor_schema="test-schema",
             input_schema=[
                 {"name": "input_data", "type": "data:test:", "required": False}
             ],
@@ -395,14 +393,115 @@ class TestDataViewSetCase(TestCase):
             descriptor_schema=self.descriptor_schema,
         )
         self.entity = Entity.objects.create(
-            collection=self.collection,
-            contributor=self.contributor,
-            descriptor_schema=self.descriptor_schema,
+            collection=self.collection, contributor=self.contributor
         )
         self.collection.set_permission(Permission.EDIT, self.contributor)
         self.proc.set_permission(Permission.VIEW, self.contributor)
         self.descriptor_schema.set_permission(Permission.VIEW, self.contributor)
 
+
+class TestDataViewSetCaseTransaction(TestDataViewSetCaseMixin, TransactionTestCase):
+    """Test background jobs."""
+
+    def test_move_to_collection(self):
+        data_in_entity = Data.objects.create(
+            contributor=self.contributor, process=self.proc
+        )
+        data_orphan = Data.objects.create(
+            contributor=self.contributor, process=self.proc
+        )
+        data_orphan.entity.data.remove(data_orphan)
+
+        collection_1 = Collection.objects.create(contributor=self.contributor)
+        collection_1.data.add(data_orphan)
+        collection_1.set_permission(Permission.EDIT, self.contributor)
+
+        collection_2 = Collection.objects.create(contributor=self.contributor)
+        collection_2.set_permission(Permission.EDIT, self.contributor)
+
+        # Assert preventing moving data in entity.
+        request = factory.post(
+            reverse("resolwe-api:data-move-to-collection"),
+            {
+                "ids": [data_in_entity.id],
+                "destination_collection": collection_2.id,
+            },
+            format="json",
+        )
+        force_authenticate(request, self.contributor)
+        response = self.move_to_collection_viewset(request)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        # Wait for the task to finish.
+        task = BackgroundTask.objects.get(pk=response.data["id"])
+        task.wait(final_statuses=["ER"])
+        task.refresh_from_db()
+        self.assertEqual(
+            task.output[0],
+            "If Data is in entity, you can only move it to another collection by moving entire entity.",
+        )
+
+        # Assert moving data not in entity.
+        request = factory.post(
+            reverse("resolwe-api:data-move-to-collection"),
+            {
+                "ids": [data_orphan.id],
+                "destination_collection": collection_2.id,
+            },
+            format="json",
+        )
+        self.assertEqual(collection_1.data.count(), 1)
+        self.assertEqual(collection_2.data.count(), 0)
+
+        force_authenticate(request, self.contributor)
+        response = self.move_to_collection_viewset(request)
+        task = BackgroundTask.objects.get(pk=response.data["id"])
+        task.wait()
+        self.assertEqual(collection_1.data.count(), 0)
+        self.assertEqual(collection_2.data.count(), 1)
+
+        # Assert preventing moving data if destination collection
+        # lacks permissions.
+        collection_1.set_permission(Permission.VIEW, self.contributor)
+        request = factory.post(
+            reverse("resolwe-api:data-move-to-collection"),
+            {
+                "ids": [data_orphan.id],
+                "destination_collection": collection_1.id,
+            },
+            format="json",
+        )
+        force_authenticate(request, self.contributor)
+        response = self.move_to_collection_viewset(request)
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+        self.assertEqual(
+            response.data["detail"],
+            "You do not have permission to perform this action.",
+        )
+
+        # It shouldn't be possible to move the data if you don't
+        # have edit permission on both collections.
+        collection_1.set_permission(Permission.EDIT, self.contributor)
+        collection_2.set_permission(Permission.VIEW, self.contributor)
+        request = factory.post(
+            reverse("resolwe-api:data-move-to-collection"),
+            {
+                "ids": [data_orphan.id],
+                "destination_collection": collection_1.id,
+            },
+            format="json",
+        )
+        force_authenticate(request, self.contributor)
+        response = self.move_to_collection_viewset(request)
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+        self.assertEqual(
+            response.data["detail"],
+            "You do not have permission to perform this action.",
+        )
+        self.assertEqual(collection_1.data.count(), 0)
+        self.assertEqual(collection_2.data.count(), 1)
+
+
+class TestDataViewSetCase(TestDataViewSetCaseMixin, TestCase):
     def test_restart(self):
         data = Data.objects.create(contributor=self.contributor, process=self.proc)
         data.status = Data.STATUS_DONE
@@ -462,11 +561,7 @@ class TestDataViewSetCase(TestCase):
         collection_2 = Collection.objects.create(
             contributor=self.user, descriptor_schema=descriptor_schema_2
         )
-        entity_2 = Entity.objects.create(
-            collection=collection_2,
-            contributor=self.user,
-            descriptor_schema=descriptor_schema_2,
-        )
+        entity_2 = Entity.objects.create(collection=collection_2, contributor=self.user)
 
         for i in range(5):
             create_kwargs = {
@@ -504,7 +599,7 @@ class TestDataViewSetCase(TestCase):
         with CaptureQueriesContext(conn) as captured_queries:
             response = self.data_viewset(request)
             self.assertEqual(len(response.data), 10)
-            self.assertEqual(len(captured_queries), 12)
+            self.assertIn(len(captured_queries), [12, 13])
 
     def test_descriptor_schema(self):
         # Descriptor schema can be assigned by slug.
@@ -707,20 +802,22 @@ class TestDataViewSetCase(TestCase):
         self.assertEqual(data.entity.id, entity.id)
         self.assertEqual(entity.collection.id, self.collection.id)
 
-        # Assign entity to None
-        data.entity = None
-        data.save()
-        request = factory.patch("/", {"collection": {"id": None}}, format="json")
+        # Assign to new entity without collection.
+        new_entity = Entity.objects.create(
+            contributor=self.contributor, name="My entity"
+        )
+        new_entity.set_permission(Permission.EDIT, self.contributor)
+        request = factory.patch("/", {"entity": {"id": new_entity.id}}, format="json")
         force_authenticate(request, self.contributor)
         response = self.data_detail_viewset(request, pk=data.pk)
-
         self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
-        self.assertEqual(
-            response.data["collection"][0],
-            "Data object can not be removed from the container.",
+        self.assertContains(
+            response,
+            "Entity must belong to the same collection as data object.",
+            status_code=status.HTTP_400_BAD_REQUEST,
         )
 
-    def test_change_collection(self):
+    def test_change_container(self):
         # Create data object. Note that an entity is created as well.
         data = Data.objects.create(
             name="Test data",
@@ -734,15 +831,15 @@ class TestDataViewSetCase(TestCase):
         )
         force_authenticate(request, self.contributor)
         response = self.data_detail_viewset(request, pk=data.pk)
-        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
-        self.assertEqual(
-            response.data["collection"][0],
-            "If Data is in entity, you can only move it to another collection by moving entire entity.",
+        self.assertContains(
+            response,
+            "Entity must belong to the same collection as data object.",
+            status_code=status.HTTP_400_BAD_REQUEST,
         )
 
         entity = data.entity
 
-        # Data can not be moved from collection.
+        # Data can not be removed from container.
         with self.assertRaises(ValidationError):
             data.move_to_entity(None)
 
@@ -754,6 +851,7 @@ class TestDataViewSetCase(TestCase):
         )
         force_authenticate(request, self.contributor)
         response = self.data_detail_viewset(request, pk=data.pk)
+
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         data.refresh_from_db()
         self.assertEqual(data.collection, self.collection)
@@ -784,103 +882,95 @@ class TestDataViewSetCase(TestCase):
 
         data.refresh_from_db()
         self.assertEqual(data.tags, collection.tags)
+        self.assertEqual(data.entity, None)
 
-    def test_move_to_collection(self):
-        data_in_entity = Data.objects.create(
-            contributor=self.contributor, process=self.proc
+        # Move it in the entity in the same collection.
+        entity = Entity.objects.create(
+            contributor=self.contributor, collection=collection
         )
-        data_orphan = Data.objects.create(
-            contributor=self.contributor, process=self.proc
+        request = factory.patch("/", {"entity": {"id": entity.pk}}, format="json")
+        force_authenticate(request, self.contributor)
+        response = self.data_detail_viewset(request, pk=data.pk)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        data.refresh_from_db()
+        self.assertEqual(data.tags, collection.tags)
+        self.assertEqual(data.entity, entity)
+
+        # Move it in another entity in the same collection.
+        another_entity = Entity.objects.create(
+            contributor=self.contributor, collection=collection
         )
-        data_orphan.entity.data.remove(data_orphan)
+        request = factory.patch(
+            "/", {"entity": {"id": another_entity.pk}}, format="json"
+        )
+        force_authenticate(request, self.contributor)
+        response = self.data_detail_viewset(request, pk=data.pk)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        data.refresh_from_db()
+        self.assertEqual(data.tags, collection.tags)
+        self.assertEqual(data.entity, another_entity)
 
-        collection_1 = Collection.objects.create(contributor=self.contributor)
-        collection_1.data.add(data_orphan)
-        collection_1.set_permission(Permission.EDIT, self.contributor)
+        # Move it to entity in another collection.
+        another_collection = Collection.objects.create(contributor=self.contributor)
+        entity.move_to_collection(another_collection)
+        another_collection.set_permission(Permission.EDIT, self.contributor)
+        entity.save()
+        entity.collection.set_permission
+        request = factory.patch("/", {"entity": {"id": entity.pk}}, format="json")
+        force_authenticate(request, self.contributor)
+        response = self.data_detail_viewset(request, pk=data.pk)
+        self.assertContains(
+            response,
+            "Entity must belong to the same collection as data object.",
+            status_code=status.HTTP_400_BAD_REQUEST,
+        )
 
-        collection_2 = Collection.objects.create(contributor=self.contributor)
-        collection_2.set_permission(Permission.EDIT, self.contributor)
+        # Try to remove it from the entity.
+        request = factory.patch("/", {"entity": None}, format="json")
+        force_authenticate(request, self.contributor)
+        response = self.data_detail_viewset(request, pk=data.pk)
+        self.assertContains(
+            response,
+            "Data object can not be removed from the container.",
+            status_code=status.HTTP_400_BAD_REQUEST,
+        )
 
-        # Assert preventing moving data in entity.
-        request = factory.post(
-            reverse("resolwe-api:data-move-to-collection"),
-            {
-                "ids": [data_in_entity.id],
-                "destination_collection": collection_2.id,
-            },
+        # Try to remove it from the collection.
+        request = factory.patch("/", {"collection": None}, format="json")
+        force_authenticate(request, self.contributor)
+        response = self.data_detail_viewset(request, pk=data.pk)
+        self.assertContains(
+            response,
+            "Data object can not be removed from the container.",
+            status_code=status.HTTP_400_BAD_REQUEST,
+        )
+
+        # Move it to another collection and entity.
+        collection = Collection.objects.create(
+            contributor=self.contributor, tags=["test"]
+        )
+        entity = Entity.objects.create(
+            contributor=self.contributor, collection=collection
+        )
+        collection.set_permission(Permission.EDIT, self.contributor)
+        request = factory.patch(
+            "/",
+            {"entity": {"id": entity.pk}, "collection": {"id": collection.pk}},
             format="json",
         )
         force_authenticate(request, self.contributor)
-        response = self.move_to_collection_viewset(request)
-
-        self.assertEqual(
-            response.data["error"],
-            "If Data is in entity, you can only move it to another collection by moving entire entity.",
-        )
-
-        # Assert moving data not in entity.
-        request = factory.post(
-            reverse("resolwe-api:data-move-to-collection"),
-            {
-                "ids": [data_orphan.id],
-                "destination_collection": collection_2.id,
-            },
-            format="json",
-        )
-
-        self.assertEqual(collection_1.data.count(), 1)
-        self.assertEqual(collection_2.data.count(), 0)
-
-        force_authenticate(request, self.contributor)
-        response = self.move_to_collection_viewset(request)
-
-        self.assertEqual(collection_1.data.count(), 0)
-        self.assertEqual(collection_2.data.count(), 1)
-
-        # Assert preventing moving data if destination collection
-        # lacks permissions.
-        collection_1.set_permission(Permission.VIEW, self.contributor)
-        request = factory.post(
-            reverse("resolwe-api:data-move-to-collection"),
-            {
-                "ids": [data_orphan.id],
-                "destination_collection": collection_1.id,
-            },
-            format="json",
-        )
-        force_authenticate(request, self.contributor)
-        response = self.move_to_collection_viewset(request)
-
-        self.assertEqual(
-            response.data["detail"],
-            "You do not have permission to perform this action.",
-        )
-
-        # It shouldn't be possible to move the data if you don't
-        # have edit permission on both collections.
-        collection_1.set_permission(Permission.EDIT, self.contributor)
-        collection_2.set_permission(Permission.VIEW, self.contributor)
-        request = factory.post(
-            reverse("resolwe-api:data-move-to-collection"),
-            {
-                "ids": [data_orphan.id],
-                "destination_collection": collection_1.id,
-            },
-            format="json",
-        )
-        force_authenticate(request, self.contributor)
-        response = self.move_to_collection_viewset(request)
-
-        self.assertEqual(
-            response.data["detail"],
-            "You do not have permission to perform this action.",
-        )
-        self.assertEqual(collection_1.data.count(), 0)
-        self.assertEqual(collection_2.data.count(), 1)
+        response = self.data_detail_viewset(request, pk=data.pk)
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        data.refresh_from_db()
+        self.assertEqual(data.tags, collection.tags)
+        self.assertEqual(data.entity, entity)
+        self.assertEqual(data.collection, collection)
 
     def test_process_is_active(self):
-        # Do not allow creating data of inactive processes
-        Process.objects.filter(slug="test-process").update(is_active=False)
+        # Do not allow creating data of inactive processes.
+        # The all_objects is important, since window function (used by default manager)
+        # is not compatible with the update statement.
+        Process.all_objects.filter(slug="test-process").update(is_active=False)
         data = {"process": {"slug": "test-process"}}
         request = factory.post("/", data, format="json")
         force_authenticate(request, self.contributor)
@@ -1327,6 +1417,26 @@ class TestCollectionViewSetCase(TestCollectionViewSetCaseCommonMixin, TestCase):
         )
         self.assertEqual(get_collection(collections, "empty")["status"], None)
 
+        # Filter by status
+        response = self.client.get(self.list_url, {"status": "OK"})
+        self.assertEqual(len(response.data), 1)
+        self.assertEqual(response.data[0]["name"], "done")
+
+        response = self.client.get(self.list_url, {"status": "ER"})
+        self.assertEqual(len(response.data), 1)
+        self.assertEqual(response.data[0]["name"], "error")
+
+        response = self.client.get(
+            self.list_url,
+            {"status__in": f"{Data.STATUS_RESOLVING},{Data.STATUS_WAITING}"},
+            format="json",
+        )
+        self.assertEqual(len(response.data), 2)
+        self.assertCountEqual(
+            [response.data[0]["name"], response.data[1]["name"]],
+            ["resolving", "waiting"],
+        )
+
 
 class TestCollectionViewSetCaseDelete(
     TestCollectionViewSetCaseCommonMixin, TransactionTestCase
@@ -1372,15 +1482,17 @@ class TestCollectionViewSetCaseDelete(
         )
 
         # Anonymous request, no permission.
+        get_anonymous_user(False)
         response = client.post(
             request_path, data={"ids": [collection1.pk]}, format="json"
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        BackgroundTask.objects.get(pk=response.data["id"]).wait()
-        self.assertTrue(Collection.objects.filter(pk=collection1.pk).exists())
-        self.assertTrue(Collection.objects.filter(pk=collection2.pk).exists())
+        self.assertContains(
+            response,
+            f"No permission to delete objects with ids {set([collection1.pk])}",
+            status_code=response.status_code,
+        )
 
-        # Anonymous request, view permission.
+        # # Anonymous request, view permission.
         collection1.set_permission(Permission.VIEW, get_anonymous_user(False))
         response = client.post(
             request_path, data={"ids": [collection1.pk]}, format="json"
@@ -1394,7 +1506,7 @@ class TestCollectionViewSetCaseDelete(
         self.assertTrue(Collection.objects.filter(pk=collection2.pk).exists())
         collection1.set_permission(Permission.NONE, get_anonymous_user(False))
 
-        # All requests are authenticated as contributor from now on.
+        # # All requests are authenticated as contributor from now on.
         client.force_authenticate(self.contributor)
 
         # No permission.
@@ -1403,8 +1515,12 @@ class TestCollectionViewSetCaseDelete(
         response = client.post(
             request_path, data={"ids": [collection1.pk, collection2.pk]}, format="json"
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        BackgroundTask.objects.get(pk=response.data["id"]).wait()
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+        self.assertContains(
+            response,
+            f"No permission to delete objects with ids {set([collection1.pk, collection2.pk])}",
+            status_code=response.status_code,
+        )
         self.assertTrue(Collection.objects.filter(pk=collection1.pk).exists())
         self.assertTrue(Collection.objects.filter(pk=collection2.pk).exists())
 
@@ -1570,17 +1686,64 @@ class EntityViewSetTestCommonMixin:
             "resolwe-api:entity-detail", kwargs={"pk": pk}
         )
 
-    def _create_data(self):
+    def _create_data(self, data_status=None):
         process = Process.objects.create(
             name="Test process",
             contributor=self.contributor,
         )
-
-        return Data.objects.create(
+        data = Data.objects.create(
             name="Test data",
             contributor=self.contributor,
             process=process,
         )
+        if data_status:
+            data.status = data_status
+            data.save()
+
+        return data
+
+
+class EntityViewSetTestTransaction(EntityViewSetTestCommonMixin, TransactionTestCase):
+    """Test background task."""
+
+    def test_move_to_collection(self):
+        source_collection = Collection.objects.create(contributor=self.contributor)
+        source_collection.set_permission(Permission.EDIT, self.contributor)
+        entity = Entity.objects.create(
+            contributor=self.contributor, collection=source_collection
+        )
+        data = self._create_data()
+        data.entity = entity
+        data.collection = source_collection
+        data.save()
+
+        destination_collection = Collection.objects.create(contributor=self.contributor)
+        destination_collection.set_permission(Permission.EDIT, self.contributor)
+
+        request = factory.post(
+            reverse("resolwe-api:entity-move-to-collection"),
+            {
+                "ids": [entity.id],
+                "source_collection": source_collection.id,
+                "destination_collection": destination_collection.id,
+            },
+            format="json",
+        )
+        self.assertEqual(source_collection.entity_set.count(), 1)
+        self.assertEqual(source_collection.data.count(), 1)
+        self.assertEqual(destination_collection.entity_set.count(), 0)
+        self.assertEqual(destination_collection.data.count(), 0)
+
+        force_authenticate(request, self.contributor)
+        response = self.move_to_collection_viewset(request)
+        task = BackgroundTask.objects.get(pk=response.data["id"])
+        task.wait()
+
+        self.assertEqual(source_collection.entity_set.count(), 0)
+        self.assertEqual(source_collection.data.count(), 0)
+        self.assertEqual(destination_collection.entity_set.count(), 1)
+        self.assertEqual(destination_collection.entity_set.first().id, entity.id)
+        self.assertEqual(destination_collection.data.first().id, data.id)
 
 
 class EntityViewSetTest(EntityViewSetTestCommonMixin, TestCase):
@@ -1593,10 +1756,7 @@ class EntityViewSetTest(EntityViewSetTestCommonMixin, TestCase):
         )
 
         for i in range(5):
-            create_kwargs = {
-                "contributor": self.contributor,
-                "descriptor_schema": self.descriptor_schema,
-            }
+            create_kwargs = {"contributor": self.contributor}
             if i < 4:
                 create_kwargs["collection"] = self.collection
             entity = Entity.objects.create(**create_kwargs)
@@ -1605,10 +1765,7 @@ class EntityViewSetTest(EntityViewSetTestCommonMixin, TestCase):
             )
 
         for i in range(5):
-            create_kwargs = {
-                "contributor": self.user,
-                "descriptor_schema": descriptor_schema_2,
-            }
+            create_kwargs = {"contributor": self.user}
             if i < 4:
                 create_kwargs["collection"] = collection_2
             entity = Entity.objects.create(**create_kwargs)
@@ -1630,7 +1787,122 @@ class EntityViewSetTest(EntityViewSetTestCommonMixin, TestCase):
         with CaptureQueriesContext(conn) as captured_queries:
             response = self.entity_list_viewset(request)
             self.assertEqual(len(response.data), 10)
-            self.assertEqual(len(captured_queries), 7)
+            self.assertIn(len(captured_queries), [7, 8])
+
+    def test_entity_status(self):
+        data_error = self._create_data(Data.STATUS_ERROR)
+        data_uploading = self._create_data(Data.STATUS_UPLOADING)
+        data_processing = self._create_data(Data.STATUS_PROCESSING)
+        data_preparing = self._create_data(Data.STATUS_PREPARING)
+        data_waiting = self._create_data(Data.STATUS_WAITING)
+        data_resolving = self._create_data(Data.STATUS_RESOLVING)
+        data_done = self._create_data(Data.STATUS_DONE)
+
+        entity = Entity.objects.create(contributor=self.contributor, name="error")
+        entity.set_permission(Permission.VIEW, get_anonymous_user())
+        entity.data.add(
+            data_error,
+            data_uploading,
+            data_processing,
+            data_preparing,
+            data_waiting,
+            data_resolving,
+            data_done,
+        )
+
+        entity = Entity.objects.create(contributor=self.contributor, name="uploading")
+        entity.set_permission(Permission.VIEW, get_anonymous_user())
+        entity.data.add(
+            data_uploading,
+            data_processing,
+            data_preparing,
+            data_waiting,
+            data_resolving,
+            data_done,
+        )
+
+        entity = Entity.objects.create(contributor=self.contributor, name="processing")
+        entity.set_permission(Permission.VIEW, get_anonymous_user())
+        entity.data.add(
+            data_processing, data_preparing, data_waiting, data_resolving, data_done
+        )
+
+        entity = Entity.objects.create(contributor=self.contributor, name="preparing")
+        entity.set_permission(Permission.VIEW, get_anonymous_user())
+        entity.data.add(data_preparing, data_waiting, data_resolving, data_done)
+
+        entity = Entity.objects.create(contributor=self.contributor, name="waiting")
+        entity.set_permission(Permission.VIEW, get_anonymous_user())
+        entity.data.add(data_waiting, data_resolving, data_done)
+
+        entity = Entity.objects.create(contributor=self.contributor, name="resolving")
+        entity.set_permission(Permission.VIEW, get_anonymous_user())
+        entity.data.add(data_resolving, data_done)
+
+        entity = Entity.objects.create(contributor=self.contributor, name="done")
+        entity.set_permission(Permission.VIEW, get_anonymous_user())
+        entity.data.add(data_done)
+
+        entity = Entity.objects.create(contributor=self.contributor, name="empty")
+        entity.set_permission(Permission.VIEW, get_anonymous_user())
+        entity.data.add()
+
+        request = factory.get("/", {}, format="json")
+        force_authenticate(request, self.contributor)
+        entities = self.entity_list_viewset(request).data
+
+        # entities = self.client.get(self.list_url).data
+
+        get_entity = lambda collections, name: next(
+            x for x in collections if x["name"] == name
+        )
+        self.assertEqual(get_entity(entities, "error")["status"], Data.STATUS_ERROR)
+        self.assertEqual(
+            get_entity(entities, "uploading")["status"], Data.STATUS_UPLOADING
+        )
+        self.assertEqual(
+            get_entity(entities, "processing")["status"], Data.STATUS_PROCESSING
+        )
+        self.assertEqual(
+            get_entity(entities, "preparing")["status"], Data.STATUS_PREPARING
+        )
+        self.assertEqual(get_entity(entities, "waiting")["status"], Data.STATUS_WAITING)
+        self.assertEqual(
+            get_entity(entities, "resolving")["status"], Data.STATUS_RESOLVING
+        )
+        self.assertEqual(get_entity(entities, "done")["status"], Data.STATUS_DONE)
+        self.assertEqual(get_entity(entities, "empty")["status"], None)
+
+        # Filter by status
+        request = factory.get(
+            "/", {"status": "OK", "collection__isnull": True}, format="json"
+        )
+        force_authenticate(request, self.contributor)
+        response = self.entity_list_viewset(request)
+        self.assertEqual(len(response.data), 1)
+        self.assertEqual(response.data[0]["name"], "done")
+
+        request = factory.get(
+            "/", {"status": "ER", "collection__isnull": True}, format="json"
+        )
+        response = self.entity_list_viewset(request)
+        self.assertEqual(len(response.data), 1)
+        self.assertEqual(response.data[0]["name"], "error")
+
+        request = factory.get(
+            "/",
+            {
+                "status__in": f"{Data.STATUS_RESOLVING},{Data.STATUS_WAITING}",
+                "collection__isnull": True,
+            },
+            format="json",
+        )
+        response = self.entity_list_viewset(request)
+        self.assertEqual(len(response.data), 2)
+        self.assertCountEqual(
+            [response.data[0]["name"], response.data[1]["name"]],
+            ["resolving", "waiting"],
+        )
 
     def test_list_filter_collection(self):
         request = factory.get("/", {}, format="json")
@@ -1680,44 +1952,6 @@ class EntityViewSetTest(EntityViewSetTestCommonMixin, TestCase):
 
         self.assertIn("can only be moved to another container.", resp.data["error"])
 
-    def test_move_to_collection(self):
-        source_collection = Collection.objects.create(contributor=self.contributor)
-        source_collection.set_permission(Permission.EDIT, self.contributor)
-        entity = Entity.objects.create(
-            contributor=self.contributor, collection=source_collection
-        )
-        data = self._create_data()
-        data.entity = entity
-        data.collection = source_collection
-        data.save()
-
-        destination_collection = Collection.objects.create(contributor=self.contributor)
-        destination_collection.set_permission(Permission.EDIT, self.contributor)
-
-        request = factory.post(
-            reverse("resolwe-api:entity-move-to-collection"),
-            {
-                "ids": [entity.id],
-                "source_collection": source_collection.id,
-                "destination_collection": destination_collection.id,
-            },
-            format="json",
-        )
-        force_authenticate(request, self.contributor)
-
-        self.assertEqual(source_collection.entity_set.count(), 1)
-        self.assertEqual(source_collection.data.count(), 1)
-        self.assertEqual(destination_collection.entity_set.count(), 0)
-        self.assertEqual(destination_collection.data.count(), 0)
-
-        self.move_to_collection_viewset(request)
-
-        self.assertEqual(source_collection.entity_set.count(), 0)
-        self.assertEqual(source_collection.data.count(), 0)
-        self.assertEqual(destination_collection.entity_set.count(), 1)
-        self.assertEqual(destination_collection.entity_set.first().id, entity.id)
-        self.assertEqual(destination_collection.data.first().id, data.id)
-
     def test_duplicate_not_auth(self):
         request = factory.post(reverse("resolwe-api:entity-duplicate"), format="json")
         response = self.duplicate_viewset(request)
@@ -1755,8 +1989,11 @@ class EntityViewSetTestDelete(EntityViewSetTestCommonMixin, TransactionTestCase)
 
         # Anonymous request, no permission.
         response = client.post(request_path, data={"ids": [entity1.pk]}, format="json")
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        BackgroundTask.objects.get(pk=response.data["id"]).wait()
+        self.assertContains(
+            response,
+            f"No permission to delete objects with ids {set([entity1.pk])}",
+            status_code=response.status_code,
+        )
         self.assertTrue(Entity.objects.filter(pk=entity1.pk).exists())
         self.assertTrue(Entity.objects.filter(pk=entity2.pk).exists())
 
@@ -1781,8 +2018,11 @@ class EntityViewSetTestDelete(EntityViewSetTestCommonMixin, TransactionTestCase)
         response = client.post(
             request_path, data={"ids": [entity1.pk, entity2.pk]}, format="json"
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        BackgroundTask.objects.get(pk=response.data["id"]).wait()
+        self.assertContains(
+            response,
+            f"No permission to delete objects with ids {set([entity1.pk, entity2.pk])}",
+            status_code=response.status_code,
+        )
         self.assertTrue(Entity.objects.filter(pk=entity1.pk).exists())
         self.assertTrue(Entity.objects.filter(pk=entity2.pk).exists())
 
