@@ -3,7 +3,6 @@ import os
 from io import StringIO
 
 import yaml
-
 from django.core.management import call_command
 from django.core.management.base import CommandError
 from django.test import TestCase as DjangoTestCase
@@ -25,10 +24,11 @@ class ProcessRegisterTest(TestCase):
     def test_process_register_all(self):
         out, err = StringIO(), StringIO()
         call_command("register", stdout=out, stderr=err)
-        self.assertTrue("Inserted test-min" in out.getvalue())
-        self.assertTrue(
-            "Skip processor test-min: newer version installed" in err.getvalue()
+        self.assertIn("Inserted test-min", out.getvalue())
+        self.assertIn(
+            "Skip processor test-min: newer version installed", err.getvalue()
         )
+        self.assertNotIn("setting is_active to True", out.getvalue())
 
         # Check that contributor gets all permissions.
         process = Process.objects.first()
@@ -37,23 +37,28 @@ class ProcessRegisterTest(TestCase):
 
         out, err = StringIO(), StringIO()
         call_command("register", stdout=out, stderr=err)
-        self.assertTrue(
-            "Skip processor test-min: same version installed" in out.getvalue()
+        self.assertIn("Skip processor test-min: same version installed", out.getvalue())
+        self.assertIn(
+            "Skip processor test-bloated: same version installed", out.getvalue()
         )
-        self.assertTrue(
-            "Skip processor test-bloated: same version installed" in out.getvalue()
-        )
-        self.assertTrue(
-            "Skip processor test-min: newer version installed" in err.getvalue()
+        self.assertIn(
+            "Skip processor test-min: newer version installed", err.getvalue()
         )
+        self.assertNotIn("setting is_active to True", out.getvalue())
 
         out, err = StringIO(), StringIO()
         call_command("register", force=True, stdout=out, stderr=err)
-        self.assertTrue("Updated test-min" in out.getvalue())
-        self.assertTrue("Updated test-bloated" in out.getvalue())
-        self.assertTrue(
-            "Skip processor test-min: newer version installed" in err.getvalue()
+        self.assertIn("Updated test-min", out.getvalue())
+        self.assertIn("Updated test-bloated", out.getvalue())
+        self.assertIn(
+            "Skip processor test-min: newer version installed", err.getvalue()
         )
+        self.assertNotIn("setting is_active to True", out.getvalue())
+        # The all_objects is important, since window function (used by the default
+        # manager) is not compatible with the update statement.
+        Process.all_objects.filter(slug="test-min").update(is_active=False)
+        call_command("register", stdout=out, stderr=err)
+        self.assertIn("Processor test-min: setting is_active to True", out.getvalue())
 
     def test_validation_of_defaults(self):
         out, err = StringIO(), StringIO()
