@@ -4,7 +4,6 @@ import datetime
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
 from django.utils.timezone import get_current_timezone
-
 from rest_framework import status
 from rest_framework.test import APIRequestFactory, force_authenticate
 
@@ -224,6 +223,49 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         )
         self._check_filter({"descriptor_schema": "999999"}, [])
 
+    def test_mixed_descriptor_schema_version(self):
+        """Test processes with mixed descriptor schema versions are shown.
+
+        When multiple data objects with different versions of the descriptor schema
+        with same slug are in the filter result both process versions must be returned.
+        """
+        descriptor_schema = DescriptorSchema.objects.create(
+            slug="test-schema",
+            version="1.4.2",
+            contributor=self.contributor,
+            schema=[
+                {
+                    "name": "company",
+                    "group": [
+                        {
+                            "name": "name",
+                            "type": "basic:string:",
+                            "required": False,
+                        },
+                        {
+                            "name": "departments",
+                            "type": "list:basic:string:",
+                            "required": False,
+                        },
+                    ],
+                }
+            ],
+        )
+        Collection.objects.create(
+            contributor=self.contributor, descriptor_schema=descriptor_schema
+        ),
+        request = factory.get("/", {}, format="json")
+        force_authenticate(request, self.admin)
+        response = self.viewset(request)
+        self.assertSetEqual(
+            {"1.0.0", "1.4.2"},
+            {
+                entry["descriptor_schema"]["version"]
+                for entry in response.data
+                if entry["descriptor_schema"] is not None
+            },
+        )
+
     def test_filter_tags(self):
         self._check_filter({"tags": "first-tag"}, self.collections[:2])
         self._check_filter({"tags": "first-tag,second-tag"}, [self.collections[1]])
@@ -313,6 +355,17 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         self._check_filter({"text": "test"}, self.collections)
         self._check_filter({"text": "user"}, [self.collections[2]])
 
+        # Test unaccented search.
+        self._check_filter({"text": "Tešt čollećtion 1"}, [self.collections[1]])
+
+        accented_collection = Collection.objects.create(
+            name="Böhm", contributor=self.admin
+        )
+        accented_collection.set_permission(Permission.VIEW, self.user)
+        self._check_filter({"text": "Bohm"}, [accented_collection])
+        self._check_filter({"text": "Böhm"}, [accented_collection])
+        accented_collection.delete()
+
         # By contributor.
         self._check_filter({"text": "joe"}, self.collections[:2])
         self._check_filter({"text": "Miller"}, self.collections[:2])
@@ -369,11 +422,21 @@ class CollectionViewSetFiltersTest(BaseViewSetFiltersTest):
         result = self._check_filter({"text": "test collection"}, self.collections)
         self.assertEqual(result.data[0]["id"], self.collections[2].pk)
 
-        # Check that ordering can be overriden.
+        # Check BAD_REQUEST status code is returned when full-text search and ordering
+        # are used at the same time.
         result = self._check_filter(
-            {"text": "test collection", "ordering": "id"}, self.collections
+            {"text": "test collection", "ordering": "id"},
+            self.collections,
+            expected_status_code=status.HTTP_400_BAD_REQUEST,
+        )
+        self.assertContains(
+            result,
+            (
+                "Ordering by full text search rank and other fields (id) is not "
+                "supported."
+            ),
+            status_code=status.HTTP_400_BAD_REQUEST,
         )
-        self.assertEqual(result.data[0]["id"], self.collections[0].pk)
 
     def test_filter_entity_count(self):
         self._check_filter(
@@ -451,13 +514,6 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
                 contributor=cls.contributor,
                 collection=cls.collection1,
                 description="My description!",
-                descriptor_schema=cls.descriptor_schema,
-                descriptor={
-                    "company": {
-                        "name": "Genialis",
-                        "departments": ["engineering", "operations"],
-                    }
-                },
                 tags=["first-tag"],
             ),
             Entity.objects.create(
@@ -465,9 +521,8 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
                 slug="test-entity-1",
                 contributor=cls.contributor,
                 collection=cls.collection2,
-                description="My favourite test entity",
-                descriptor_schema=cls.descriptor_schema,
                 tags=["first-tag", "second-tag"],
+                description="My favourite test entity",
             ),
             Entity.objects.create(
                 name="User test entity",
@@ -545,12 +600,6 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         self._check_filter({"description__contains": "Favourite"}, [])
         self._check_filter({"description__contains": "420"}, [])
 
-    def test_filter_descriptor_schema(self):
-        self._check_filter(
-            {"descriptor_schema": str(self.descriptor_schema.pk)}, self.entities[:2]
-        )
-        self._check_filter({"descriptor_schema": "999999"}, [])
-
     def test_filter_tags(self):
         self._check_filter({"tags": "first-tag"}, self.entities[:2])
         self._check_filter({"tags": "first-tag,second-tag"}, [self.entities[1]])
@@ -677,10 +726,6 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         self._check_filter({"text": "my description"}, [self.entities[0]])
         self._check_filter({"text": "user"}, self.entities[2:])
 
-        # By descriptor.
-        self._check_filter({"text": "genialis"}, [self.entities[0]])
-        self._check_filter({"text": "engineering"}, [self.entities[0]])
-
         # By mixed fields.
         self._check_filter({"text": "test joe"}, self.entities[:2])
         self._check_filter({"text": "joe my description"}, [self.entities[0]])
@@ -711,11 +756,21 @@ class EntityViewSetFiltersTest(BaseViewSetFiltersTest):
         result = self._check_filter({"text": "test entity"}, self.entities)
         self.assertEqual(result.data[0]["id"], self.entities[2].pk)
 
-        # Check that ordering can be overriden.
+        # Check BAD_REQUEST status code is returned when full-text search and ordering
+        # are used at the same time.
         result = self._check_filter(
-            {"text": "test entity", "ordering": "id"}, self.entities
+            {"text": "test entity", "ordering": "id"},
+            self.entities,
+            expected_status_code=status.HTTP_400_BAD_REQUEST,
+        )
+        self.assertContains(
+            result,
+            (
+                "Ordering by full text search rank and other fields (id) is not "
+                "supported."
+            ),
+            status_code=status.HTTP_400_BAD_REQUEST,
         )
-        self.assertEqual(result.data[0]["id"], self.entities[0].pk)
 
     def test_order_by_collection_name(self):
         """Check entities can be ordered by the collection name."""
@@ -1041,6 +1096,71 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
             self.data[2:],
         )
 
+    def test_mixed_process_version(self):
+        """Test processes with mixed versions are shown.
+
+        When multiple data objects with different versions of the process with same
+        slug are in the filter result both process versions must be returned.
+        """
+        # Create data object with never version of the test-process-1 process.
+        process = Process.objects.create(
+            slug="test-process-1", version="2.0.0", contributor=self.contributor
+        )
+        Data.objects.create(
+            contributor=self.contributor, collection=self.collection1, process=process
+        ),
+        request = factory.get("/", {"collection": self.collection1.pk}, format="json")
+        force_authenticate(request, self.admin)
+        response = self.viewset(request)
+        self.assertEqual(len(response.data), 2)
+        self.assertCountEqual(
+            ["1.0.0", "2.0.0"],
+            [entry["process"]["version"] for entry in response.data],
+        )
+
+    def test_mixed_descriptor_schema_version(self):
+        """Test processes with mixed descriptor schema versions are shown.
+
+        When multiple data objects with different versions of the descriptor schema
+        with same slug are in the filter result both process versions must be returned.
+        """
+        descriptor_schema = DescriptorSchema.objects.create(
+            slug="test-schema",
+            version="2.0.0",
+            contributor=self.contributor,
+            schema=[
+                {
+                    "name": "company",
+                    "group": [
+                        {
+                            "name": "name",
+                            "type": "basic:string:",
+                            "required": False,
+                        },
+                        {
+                            "name": "departments",
+                            "type": "list:basic:string:",
+                            "required": False,
+                        },
+                    ],
+                }
+            ],
+        )
+        Data.objects.create(
+            contributor=self.contributor,
+            collection=self.collection1,
+            process=self.proc1,
+            descriptor_schema=descriptor_schema,
+        ),
+        request = factory.get("/", {"collection": self.collection1.pk}, format="json")
+        force_authenticate(request, self.admin)
+        response = self.viewset(request)
+        self.assertEqual(len(response.data), 2)
+        self.assertCountEqual(
+            ["1.0.0", "2.0.0"],
+            [entry["descriptor_schema"]["version"] for entry in response.data],
+        )
+
     def test_filter_collection_name(self):
         self._check_filter({"collection__name": "My collection"}, [self.data[0]])
         self._check_filter({"collection__name__icontains": "Collection"}, self.data[:2])
@@ -1162,9 +1282,21 @@ class DataViewSetFiltersTest(BaseViewSetFiltersTest):
         result = self._check_filter({"text": "data"}, self.data)
         self.assertEqual(result.data[0]["id"], self.data[2].pk)
 
-        # Check that ordering can be overriden.
-        result = self._check_filter({"text": "data", "ordering": "id"}, self.data)
-        self.assertEqual(result.data[0]["id"], self.data[0].pk)
+        # Check BAD_REQUEST status code is returned when full-text search and ordering
+        # are used at the same time.
+        result = self._check_filter(
+            {"text": "data", "ordering": "id"},
+            self.data,
+            expected_status_code=status.HTTP_400_BAD_REQUEST,
+        )
+        self.assertContains(
+            result,
+            (
+                "Ordering by full text search rank and other fields (id) is not "
+                "supported."
+            ),
+            status_code=status.HTTP_400_BAD_REQUEST,
+        )
 
     def test_ordering_entity_name(self):
         """Check that data can be ordered by the entity name."""
@@ -1326,6 +1458,16 @@ class ProcessViewSetFiltersTest(BaseViewSetFiltersTest):
         self._check_filter({"type": "data:alignment:bam"}, [self.proc_1])
         self._check_filter({"type": "data:expression"}, [self.proc_2])
 
+    def test_return_all_versions(self):
+        new_version = Process.objects.create(
+            contributor=self.contributor,
+            type="data:expression:",
+            category="analyses:",
+            scheduling_class="IN",
+            version="3.0.0",
+        )
+        self._check_filter({"type": "data:expression"}, [self.proc_2, new_version])
+
     def test_scheduling_class(self):
         self._check_filter({"scheduling_class": "BA"}, [self.proc_1])
         self._check_filter({"scheduling_class": "IN"}, [self.proc_2])
