@@ -18,14 +18,15 @@ import json
 from pathlib import Path
 from typing import Dict
 
-from versionfield.version import Version
-
 from django.apps import apps
 from django.conf import settings
 from django.db import models
 
+from resolwe.flow.models.fields import Version
+
 from .decorators import retry
 from .iterators import iterate_dict, iterate_fields, iterate_schema
+from .singleton import Singleton
 
 __all__ = (
     "dict_dot",
@@ -35,6 +36,7 @@ __all__ = (
     "iterate_fields",
     "iterate_schema",
     "retry",
+    "Singleton",
 )
 
 
