@@ -1,17 +1,8 @@
 """Data viewset."""
 
-from drf_spectacular.utils import extend_schema
-
 from django.db.models import Prefetch
-
-from rest_framework import (
-    exceptions,
-    mixins,
-    permissions,
-    serializers,
-    status,
-    viewsets,
-)
+from drf_spectacular.utils import extend_schema
+from rest_framework import exceptions, mixins, serializers, status, viewsets
 from rest_framework.decorators import action
 from rest_framework.response import Response
 
@@ -21,6 +12,7 @@ from resolwe.flow.models.utils import fill_with_defaults
 from resolwe.flow.serializers import DataSerializer
 from resolwe.flow.utils import get_data_checksum
 from resolwe.observers.mixins import ObservableMixin
+from resolwe.observers.views import BackgroundTaskSerializer
 from resolwe.permissions.loader import get_permissions_class
 from resolwe.permissions.mixins import ResolwePermissionsMixin
 from resolwe.permissions.models import Permission, PermissionModel
@@ -34,19 +26,7 @@ from .mixins import (
     ResolweCreateModelMixin,
     ResolweUpdateModelMixin,
 )
-from .utils import get_collection_for_user
-
-
-class IsStaffuser(permissions.BasePermission):
-    """Allow access only to staff users."""
-
-    message = "Only staff users are allowed."
-
-    def has_permission(self, request, view):
-        """Return true when request is allowed."""
-        return bool(
-            request.user and request.user.is_authenticated and request.user.is_staff
-        )
+from .utils import IsStaffuser, get_collection_for_user
 
 
 class MoveDataToCollectionSerializer(serializers.Serializer):
@@ -77,8 +57,8 @@ class DataViewSet(
     """API view for :class:`Data` objects."""
 
     qs_permission_model = PermissionModel.objects.select_related("user", "group")
-    qs_descriptor_schema = DescriptorSchema.objects.select_related("contributor")
-    qs_process = Process.objects.select_related("contributor")
+    qs_descriptor_schema = DescriptorSchema.all_objects.select_related("contributor")
+    qs_process = Process.all_objects.select_related("contributor")
     queryset = Data.objects.select_related("contributor").prefetch_related(
         Prefetch("collection", queryset=BaseCollectionViewSet.queryset),
         Prefetch("descriptor_schema", queryset=qs_descriptor_schema),
@@ -199,9 +179,11 @@ class DataViewSet(
         dst_collection = get_collection_for_user(dst_collection_id, request.user)
 
         queryset = self._get_data(request.user, ids)
-        queryset.move_to_collection(dst_collection)
-
-        return Response()
+        task = queryset.move_to_collection(dst_collection, request.user)
+        return Response(
+            status=status.HTTP_200_OK,
+            data=BackgroundTaskSerializer(task).data,
+        )
 
     def _get_data(self, user, ids):
         """Return data objects queryset based on provided ids."""
