@@ -1,9 +1,7 @@
 """Mixins used in Resolwe Viewsets."""
 
-from drf_spectacular.utils import extend_schema
-
 from django.db import IntegrityError, transaction
-
+from drf_spectacular.utils import extend_schema
 from rest_framework import exceptions, mixins, serializers, status
 from rest_framework.decorators import action
 from rest_framework.generics import get_object_or_404
@@ -12,7 +10,11 @@ from rest_framework.response import Response
 
 from resolwe.observers.models import BackgroundTask
 from resolwe.observers.views import BackgroundTaskSerializer
-from resolwe.permissions.models import Permission, get_anonymous_user
+from resolwe.permissions.models import (
+    Permission,
+    PermissionInterface,
+    get_anonymous_user,
+)
 from resolwe.permissions.utils import assign_contributor_permissions
 
 
@@ -45,16 +47,19 @@ class ResolweCreateModelMixin(mixins.CreateModelMixin):
         return get_anonymous_user() if user.is_anonymous else user
 
     def define_contributor(self, request):
-        """Define contributor by adding it to request.data."""
-        request.data["contributor"] = self.resolve_user(request.user).pk
+        """Define contributor by adding it to request.data.
+
+        When data is a list, iterate over it and add the contributor to each entry.
+        """
+        contributor_pk = self.resolve_user(request.user).pk
+        for entry in request.data if isinstance(request.data, list) else [request.data]:
+            entry["contributor"] = contributor_pk
 
     def create(self, request, *args, **kwargs):
         """Create a resource."""
         self.define_contributor(request)
-
         try:
             return super().create(request, *args, **kwargs)
-
         except IntegrityError as ex:
             return Response({"error": str(ex)}, status=status.HTTP_409_CONFLICT)
 
@@ -62,7 +67,7 @@ class ResolweCreateModelMixin(mixins.CreateModelMixin):
         """Create a resource."""
         with transaction.atomic():
             instance = serializer.save()
-            if hasattr(instance, "permission_group"):
+            if isinstance(instance, PermissionInterface):
                 # Assign all permissions to the object contributor when object is not
                 # in container.
                 if not instance.in_container():
@@ -107,7 +112,7 @@ class ResolweBackgroundDuplicateMixin:
             )
 
         duplicate_method = getattr(to_duplicate, self.BACKGROUND_DUPLICATE_METHOD)
-        task = duplicate_method(contributor=request.user)
+        task = duplicate_method(request_user=request.user)
 
         return Response(
             status=status.HTTP_200_OK,
@@ -154,6 +159,11 @@ class ResolweBackgroundDeleteMixin(mixins.DestroyModelMixin):
         serializer = DeleteSerializer(data=request.data)
         serializer.is_valid(raise_exception=True)
         ids = set(serializer.validated_data["ids"])
+
+        # The list of ids must not be empty.
+        if not ids:
+            raise exceptions.ValidationError("No ids provided.")
+
         queryset = self.get_queryset()
         if not getattr(queryset, self.BACKGROUND_DELETE_METHOD, None):
             raise exceptions.ValidationError(
@@ -172,6 +182,14 @@ class ResolweBackgroundDeleteMixin(mixins.DestroyModelMixin):
             raise exceptions.PermissionDenied(
                 f"No permission to delete objects with ids {joined_ids}."
             )
+
+        # User can not delete any object.
+        if not can_delete_ids:
+            raise exceptions.PermissionDenied(
+                f"No permission to delete objects with ids {ids}."
+            )
+
+        # Start the background task.
         task = getattr(can_delete, self.BACKGROUND_DELETE_METHOD, None)(request.user)
         return Response(
             status=status.HTTP_200_OK,
