@@ -6,13 +6,13 @@ from typing import Any, Iterable, List, Optional, Set, Tuple
 
 from asgiref.sync import async_to_sync
 from channels.layers import get_channel_layer
-
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.db import models, transaction
 from django.db.models import Count, Q
 from django.db.models.query import QuerySet
 
+from resolwe.flow.models.base import BaseManagerWithoutVersion
 from resolwe.permissions.models import Permission, PermissionObject
 
 from .protocol import GROUP_SESSIONS, TYPE_ITEM_UPDATE, ChangeType, ChannelsMessage
@@ -52,6 +52,8 @@ class BackgroundTask(Observable):
         (STATUS_ERROR, "Error"),
     )
 
+    objects = BaseManagerWithoutVersion()
+
     #: task start date and time
     started = models.DateTimeField(blank=True, null=True, db_index=True)
 
@@ -307,16 +309,20 @@ class Subscription(models.Model):
                 )
                 self.observers.add(observer)
 
+    @transaction.atomic
     def delete(self):
         """Delete the given subscription.
 
         Delete all observers with no remaining subscriptions.
+
+        This method is marked atomic, so no new subsciptions are added to the observers
+        we are deleting in the meantime causing IntegrityError.
         """
-        # Find related observers with only one remaining subscription
-        # (it must be this one) and delete them first.
+        # First find observers with only this subscription and delete them.
         Observer.objects.annotate(subs=Count("subscriptions")).filter(
             subscriptions=self.pk, subs=1
         ).delete()
+        # Now delete the subscription itself.
         super().delete()
 
     @classmethod
