@@ -3,18 +3,23 @@
 import logging
 from enum import IntEnum
 from functools import reduce
-from typing import Iterable, List, Optional, Union
+from typing import Generic, Iterable, List, Optional, TypeVar, Union
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
-from django.contrib.auth.models import Group, User
+from django.contrib.auth.models import AnonymousUser, Group, User
 from django.db import models, transaction
 
 from resolwe.observers.protocol import post_permission_changed, pre_permission_changed
 
+UserOrGroup = Union[User, Group]
+M = TypeVar("M", bound=models.Model)
+MP = TypeVar("MP", bound="PermissionInterface")
+Q = TypeVar("Q", bound=models.QuerySet)
+
+
 logger = logging.getLogger(__name__)
 PermissionList = List["Permission"]
-UserOrGroup = Union[User, Group]
 
 
 # Get and store the anonymous user for later use to avoid hits to the database.
@@ -118,6 +123,169 @@ class Permission(IntEnum):
         return self.name.lower()
 
 
+class PermissionQuerySet(models.QuerySet[MP]):
+    """Queryset with methods that simlify filtering by permissions."""
+
+    def _check_permissions(self):
+        """Raise exception if the model does not support permissions."""
+        if not issubclass(self.model, PermissionInterface):
+            raise TypeError(
+                f"Permissions are not supported on model `{self.model._meta.label}`."
+            )
+
+    def remove_delete_markers(self) -> "PermissionQuerySet":
+        """Remote delete markers from the queryset.
+
+        Due to the nature of the queryset use the method only on small querysets.
+        """
+        return self.filter(pk__in=list(self.values_list("pk", flat=True))).exclude(
+            _value__isnull=True
+        )
+
+    def _filter_by_permission(
+        self,
+        user: Optional[User],
+        groups: models.QuerySet,
+        permission: "Permission",
+        public: bool = True,
+        with_superuser: bool = True,
+    ) -> "PermissionQuerySet":
+        """Filter queryset by permissions.
+
+        This is a generic method that is called in public methods.
+
+        :attr user: the user which permissions should be considered.
+
+        :attr groups: the groups which permissions should be considered.
+
+        :attr permission: the lowest permission entity must have.
+
+        :attr public: when True consider also public permission.
+
+        :attr with_superuser: when false treat superuser as reguar user.
+        """
+
+        # Skip filtering for superuser when with_superuser is set.
+        if user is not None and user.is_superuser and with_superuser:
+            return self
+
+        filters = dict()
+        permission_group_path = self.model.permission_group_path()
+
+        if user:
+            filters["user"] = models.Q(
+                **{
+                    f"{permission_group_path}__permissions__user": user,
+                    f"{permission_group_path}__permissions__value__gte": permission,
+                }
+            )
+
+        if public:
+            filters["public"] = models.Q(
+                **{
+                    f"{permission_group_path}__permissions__user": get_anonymous_user(),
+                    f"{permission_group_path}__permissions__value__gte": permission,
+                }
+            )
+        if groups:
+            filters["groups"] = models.Q(
+                **{
+                    f"{permission_group_path}__permissions__group__in": groups.values_list(
+                        "pk", flat=True
+                    ),
+                    f"{permission_group_path}__permissions__value__gte": permission,
+                }
+            )
+
+        # List here is needed otherwise more joins are performed on the query
+        # bellow. Some Django queries (for example ExpressionLateralJoin) do
+        # not like that and will fail without evaluating the ids query first.
+        ids = list(
+            self.filter(
+                reduce(lambda filters, filter: filters | filter, filters.values())
+            )
+            .distinct()
+            .values_list("pk", flat=True)
+        )
+        return self.filter(id__in=ids)
+
+    def filter_for_user(
+        self,
+        user: User | AnonymousUser,
+        permission: Permission = Permission.VIEW,
+        use_groups: bool = True,
+        public: bool = True,
+        with_superuser: bool = True,
+    ) -> "PermissionQuerySet":
+        """Filter objects for user.
+
+        :attr user: the user which permissions should be considered.
+
+        :attr permission: the lowest permission entity must have.
+
+        :attr use_groups: when True consider also permissions of the user groups.
+
+        :attr public: when True consider also public permission.
+
+        :attr with_superuser: when false treat superuser as reguar user.
+        """
+
+        self._check_permissions()
+
+        from resolwe.permissions.utils import get_user  # Circular import
+
+        user = get_user(user)
+        groups = user.groups.all() if use_groups else user.groups.none()
+
+        return self._filter_by_permission(
+            user, groups, permission, public, with_superuser
+        )
+
+    def set_permission(self, permission: Permission, user_or_group: UserOrGroup):
+        """Set the permission on the objects in the queryset."""
+        from resolwe.permissions.utils import get_identity  # Circular import
+
+        self._check_permissions()
+
+        if any(datum for datum in self if not datum.can_set_permission()):
+            ids = self.values_list("pk", flat=True)
+            raise RuntimeError(
+                f"The permissions can not be set on objects with ids {ids} of type {self.model}."
+            )
+        entity, entity_type = get_identity(user_or_group)
+        for permission_group_id in self.values_list(
+            self.model.permission_group_path(), flat=True
+        ).distinct():
+            PermissionModel.all_objects.update_or_create(
+                **{"permission_group_id": permission_group_id, entity_type: entity},
+                defaults={"value": permission.value},
+            )
+
+
+class PermissionManager(models.Manager[M], Generic[M, Q]):
+    """The manager used for permission objects."""
+
+    # The queryset to use.
+    QuerySet: type[Q] = PermissionQuerySet  # type: ignore
+
+    def get_queryset(self) -> Q:
+        """Return the queryset."""
+        return self.QuerySet(model=self.model, using=self._db)
+
+    def filter_for_user(
+        self,
+        user: User | AnonymousUser,
+        permission: Permission = Permission.VIEW,
+        use_groups: bool = True,
+        public: bool = True,
+        with_superuser: bool = True,
+    ) -> Q:
+        """Filter the objects for user."""
+        return self.get_queryset().filter_for_user(
+            user, permission, use_groups, public, with_superuser
+        )
+
+
 class PositivePermissionsManager(models.Manager):
     """Return only PermissionModels with values greater than 0."""
 
@@ -132,11 +300,6 @@ class PermissionModel(models.Model):
     Exactly one of fields user/group must be non-null.
     """
 
-    # Access all permissions, including the ones with 0 permission.
-    all_objects = models.Manager()
-    # By default return only positive permissions.
-    objects = PositivePermissionsManager()
-
     #: permission value
     value = models.PositiveSmallIntegerField()
 
@@ -158,6 +321,12 @@ class PermissionModel(models.Model):
         "PermissionGroup", on_delete=models.CASCADE, related_name="permissions"
     )
 
+    # Access all permissions, including the ones with 0 permission.
+    all_objects = models.Manager()
+
+    # By default return only positive permissions.
+    objects = PositivePermissionsManager()
+
     class Meta:
         """Define constraints enforced on the model.
 
@@ -233,7 +402,7 @@ class PermissionGroup(models.Model):
         entity, entity_name = get_identity(user_or_group)
 
         # Superuser has all the permissions.
-        if entity_name == "user" and entity.is_superuser:
+        if entity_name == "user" and entity.is_superuser:  # type: ignore
             return Permission.highest()
 
         permission_model = self.permissions.filter(**{entity_name: entity}).first()
@@ -274,148 +443,45 @@ class PermissionGroup(models.Model):
         return list(Group.objects.filter(filter).distinct())
 
 
-class PermissionQuerySet(models.QuerySet):
-    """Queryset with methods that simlify filtering by permissions."""
+class PermissionInterface(models.Model):
+    """The abstract model that defines permission interface.
 
-    def _filter_by_permission(
-        self,
-        user: Optional[User],
-        groups: models.QuerySet,
-        permission: Permission,
-        public: bool = True,
-        with_superuser: bool = True,
-    ) -> models.QuerySet:
-        """Filter queryset by permissions.
-
-        This is a generic method that is called in public methods.
-
-        :attr user: the user which permissions should be considered.
-
-        :attr groups: the groups which permissions should be considered.
-
-        :attr permission: the lowest permission entity must have.
-
-        :attr public: when True consider also public permission.
-
-        :attr with_superuser: when false treat superuser as reguar user.
-        """
-
-        # Skip filtering for superuser when with_superuser is set.
-        if user is not None and user.is_superuser and with_superuser:
-            return self
-
-        # Handle special case of Storage and Relation.
-        filters_prefix = ""
-        if self.model._meta.label == "flow.Storage":
-            filters_prefix = "data__"
-
-        filters = dict()
-        if user:
-            filters["user"] = models.Q(
-                **{
-                    f"{filters_prefix}permission_group__permissions__user": user,
-                    f"{filters_prefix}permission_group__permissions__value__gte": permission,
-                }
-            )
-
-        if public:
-            filters["public"] = models.Q(
-                **{
-                    f"{filters_prefix}permission_group__permissions__user": get_anonymous_user(),
-                    f"{filters_prefix}permission_group__permissions__value__gte": permission,
-                }
-            )
-        if groups:
-            filters["groups"] = models.Q(
-                **{
-                    f"{filters_prefix}permission_group__permissions__group__in": groups.values_list(
-                        "pk", flat=True
-                    ),
-                    f"{filters_prefix}permission_group__permissions__value__gte": permission,
-                }
-            )
-
-        # List here is needed otherwise more joins are performed on the query
-        # bellow. Some Django queries (for example ExpressionLateralJoin) do
-        # not like that and will fail without evaluating the ids query first.
-        ids = list(
-            self.filter(
-                reduce(lambda filters, filter: filters | filter, filters.values())
-            )
-            .distinct()
-            .values_list("id", flat=True)
-        )
-        return self.filter(id__in=ids)
-
-    def filter_for_user(
-        self,
-        user: User,
-        permission: Permission = Permission.VIEW,
-        use_groups: bool = True,
-        public: bool = True,
-        with_superuser: bool = True,
-    ) -> models.QuerySet:
-        """Filter objects for user.
-
-        :attr user: the user which permissions should be considered.
-
-        :attr permission: the lowest permission entity must have.
-
-        :attr use_groups: when True consider also permissions of the user groups.
-
-        :attr public: when True consider also public permission.
-
-        :attr with_superuser: when false treat superuser as reguar user.
-        """
-
-        from resolwe.permissions.utils import get_user  # Circular import
-
-        user = get_user(user)
-        groups = user.groups.all() if use_groups else user.groups.none()
-
-        return self._filter_by_permission(
-            user, groups, permission, public, with_superuser
-        )
-
-    def set_permission(self, permission: Permission, user_or_group: UserOrGroup):
-        """Set the permission on the objects in the queryset."""
-        from resolwe.permissions.utils import get_identity  # Circular import
-
-        entity, entity_type = get_identity(user_or_group)
-
-        for permission_group_id in self.values_list(
-            "permission_group", flat=True
-        ).distinct():
-            PermissionModel.all_objects.update_or_create(
-                **{"permission_group_id": permission_group_id, entity_type: entity},
-                defaults={"value": permission.value},
-            )
-
-
-class PermissionObject(models.Model):
-    """Base permission object.
-
-    Every object that has permissions must inherit from this one.
+    When a model level permissions are needed, the model mus inherit from this class.
     """
 
+    #: custom manager with permission filtering methods.
+    objects: PermissionManager = PermissionManager()
+
     class Meta:
-        """Make this class abstract so no new table is created for it."""
+        """Make a class abstract."""
 
         abstract = True
 
-    #: permission group for the object
-    permission_group = models.ForeignKey(
-        PermissionGroup, on_delete=models.CASCADE, related_name="%(class)s", null=True
-    )
+    @property
+    def permission_group(self) -> PermissionGroup:
+        """Return a permission group object."""
+        raise NotImplementedError("The method permission_group must be implemented.")
 
-    #: custom manager with permission filtering methods.
-    objects = PermissionQuerySet.as_manager()
+    @classmethod
+    def permission_proxy(cls) -> Optional[str]:
+        """Return the path to the permission permission proxy.
 
-    def __init__(self, *args, **kwargs):
-        """Initialize."""
-        # The properties used to determine if object is in container.
-        self._container_properties = ("collection", "entity")
-        super().__init__(*args, **kwargs)
+        When None, the model has no permission proxy.
+        """
+        raise NotImplementedError(
+            "The method permission_group_path must be implemented."
+        )
+
+    @classmethod
+    def permission_group_path(cls) -> str:
+        """Return the path to the permission group."""
+        base_path = "permission_group"
+        proxy = cls.permission_proxy()
+        return base_path if proxy is None else f"{proxy}__{base_path}"
+
+    def can_set_permission(self) -> bool:
+        """Can permission be set on this object."""
+        return self.permission_proxy is None
 
     def is_owner(self, user: User) -> bool:
         """Return if user is the owner of this instance."""
@@ -437,15 +503,13 @@ class PermissionObject(models.Model):
         :raises RuntimeError: when given object has no permission_group or is
             contained in a container.
         """
-        if self.in_container():
+        if not self.can_set_permission():
             raise RuntimeError(
-                f"The permissions can not be set on object {self} ({self._meta.label}) in container."
+                f"The permissions can not be set on object {self} with id {self.pk} ({self._meta.label})."
             )
 
         pre_permission_changed.send(sender=type(self), instance=self)
-
         self.permission_group.set_permission(permission, user_or_group)
-
         post_permission_changed.send(sender=type(self), instance=self)
 
     def get_permission(self, user_or_group: UserOrGroup) -> Permission:
@@ -486,6 +550,34 @@ class PermissionObject(models.Model):
         """
         return self.permission_group.groups_with_permission(permission)
 
+
+class PermissionObject(PermissionInterface):
+    """Base permission object.
+
+    Every object that has permissions must inherit from this one.
+    """
+
+    #: permission group for the object
+    permission_group = models.ForeignKey(
+        PermissionGroup, on_delete=models.CASCADE, related_name="%(class)s", null=True
+    )
+
+    class Meta:
+        """Make this class abstract so no new table is created for it."""
+
+        abstract = True
+
+    def __init__(self, *args, **kwargs):
+        """Initialize."""
+        # The properties used to determine if object is in container.
+        self._container_properties = ("collection", "entity")
+        super().__init__(*args, **kwargs)
+
+    @classmethod
+    def permission_proxy(cls) -> Optional[str]:
+        """Return the path where permission_group model is accessible."""
+        return None
+
     @property
     def topmost_container(self) -> Optional[models.Model]:
         """Get the top-most container of the object.
@@ -511,6 +603,10 @@ class PermissionObject(models.Model):
         """Return if object lies in a container."""
         return self.topmost_container is not None
 
+    def can_set_permission(self) -> bool:
+        """Can permissions be set on this object."""
+        return not self.in_container()
+
     def save(self, *args, **kwargs):
         """Set the permission_group property of the object.
 
