@@ -1,7 +1,6 @@
 # pylint: disable=missing-docstring
 from django.contrib.auth import get_user_model
 from django.test import TransactionTestCase as DjangoTransactionTestCase
-
 from rest_framework import status
 
 from resolwe.flow.models import Collection
@@ -11,7 +10,7 @@ from resolwe.permissions.models import get_anonymous_user
 from resolwe.test import ResolweAPITestCase, TransactionResolweAPITestCase
 
 MESSAGES = {
-    "NOT_FOUND": "Not found.",
+    "NOT_FOUND": "No Collection matches the given query.",
     "NO_PERMISSION": "You do not have permission to perform this action.",
 }
 
@@ -206,7 +205,8 @@ class CollectionTestCaseDelete(
     background task. The tests are in a separate class not to slow down the other tests.
     """
 
-    def _pre_setup(self, *args, **kwargs):
+    @classmethod
+    def _pre_setup(cls, *args, **kwargs):
         """Delete all previously created users.
 
         The public user is created in the migrations and recreated during test case,
