@@ -8,7 +8,6 @@ from django.contrib.auth.models import AnonymousUser
 from django.test import TransactionTestCase as DjangoTransactionTestCase
 from django.urls import reverse
 from django.utils.timezone import now
-
 from rest_framework import exceptions, status
 from rest_framework.test import APIClient
 
@@ -22,8 +21,7 @@ from resolwe.test import ResolweAPITestCase, TransactionResolweAPITestCase
 DATE_FORMAT = r"%Y-%m-%dT%H:%M:%S.%f"
 
 MESSAGES = {
-    "NOT_FOUND": "Not found.",
-    # 'NO_PERMISSION': 'You do not have permission to perform this action.',
+    "NOT_FOUND": "No Data matches the given query.",
     "ONE_ID_REQUIRED": "Exactly one id required on create.",
 }
 
@@ -620,7 +618,8 @@ class DataTestCase(DataTestCaseCommonMixin, ResolweAPITestCase):
 class DataTestCaseDelete(
     DataTestCaseCommonMixin, TransactionResolweAPITestCase, DjangoTransactionTestCase
 ):
-    def _pre_setup(self, *args, **kwargs):
+    @classmethod
+    def _pre_setup(cls, *args, **kwargs):
         """Delete all previously created users.
 
         The public user is created in the migrations and recreated during test case,
@@ -645,8 +644,11 @@ class DataTestCaseDelete(
         response = self.client.post(
             request_path, data={"ids": [self.data1.pk]}, format="json"
         )
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        BackgroundTask.objects.get(pk=response.data["id"]).wait()
+        self.assertContains(
+            response,
+            f"No permission to delete objects with ids {set([self.data1.pk])}",
+            status_code=response.status_code,
+        )
         self.assertTrue(Data.objects.filter(pk=1).exists())
         self.assertTrue(Data.objects.filter(pk=2).exists())
 
@@ -673,7 +675,11 @@ class DataTestCaseDelete(
         response = self.client.post(
             request_path, data={"ids": [self.data1.pk, self.data2.pk]}, format="json"
         )
-        BackgroundTask.objects.get(pk=response.data["id"]).wait()
+        self.assertContains(
+            response,
+            "No permission to delete objects with ids",
+            status_code=response.status_code,
+        )
         self.assertTrue(Data.objects.filter(pk=1).exists())
         self.assertTrue(Data.objects.filter(pk=2).exists())
 
