@@ -8,16 +8,16 @@ Permissions utils
 
 """
 
-from typing import Optional, Tuple, Union
+from typing import Optional, Tuple, Type, Union
 
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import AnonymousUser, Group, User
 from django.db import models, transaction
-
 from rest_framework import exceptions
 
 from resolwe.permissions.models import (
     Permission,
+    PermissionInterface,
     PermissionModel,
     UserOrGroup,
     get_anonymous_user,
@@ -49,7 +49,7 @@ def set_permission(
     obj.set_permission(permission, user_or_group)
 
 
-def get_user(user: User) -> User:
+def get_user(user: User | AnonymousUser) -> User:
     """Get the same user or anonymous one when user is not authenticated.
 
     :raises django.core.exceptions.ObjectDoesNotExist: when user is not
@@ -61,10 +61,15 @@ def get_user(user: User) -> User:
         return get_anonymous_user()
 
 
-def model_has_permissions(obj: models.Model) -> bool:
+def model_has_permissions(obj: Union[models.Model, Type[models.Model]]) -> bool:
     """Check whether model has object level permissions."""
-    additional_labels = ["flow.Storage", "flow.AnnotationValue"]
-    return hasattr(obj, "permission_group") or obj._meta.label in additional_labels
+
+    try:
+        return isinstance(obj, PermissionInterface) or issubclass(
+            obj, PermissionInterface
+        )
+    except TypeError:
+        return False
 
 
 def get_identity(user_group: Union[Group, User]) -> Tuple[UserOrGroup, str]:
