@@ -1,4 +1,5 @@
 """Utility methods for receiving/sending data over socket."""
+
 import asyncio
 import functools
 import json
@@ -26,6 +27,33 @@ from typing import (
 logger = logging.getLogger(__name__)
 
 
+PeerIdentity = bytes
+MessageDataType = TypeVar("MessageDataType")
+ResponseDataType = TypeVar("ResponseDataType")
+
+
+class MessageProcessingEventType(Enum):
+    """Type of the event in the message processing pipeline."""
+
+    MESSAGE_RECEIVED = "MR"
+    MESSAGE_PROCESSING_STARTED = "MPS"
+    MESSAGE_PROCESSING_FINISHED = "MPF"
+    PREPARATION_FINISHED = "PF"
+
+
+class MessageProcessingCallback:
+    """Event in the message processing pipeline."""
+
+    def event(
+        self,
+        event_type: MessageProcessingEventType,
+        message: "Message",
+        peer_identity: PeerIdentity,
+    ):
+        """Event handler."""
+        raise NotImplementedError("Method must be implemented in the subclass.")
+
+
 @unique
 class MessageType(Enum):
     """Type of the message."""
@@ -40,6 +68,7 @@ class ResponseStatus(Enum):
 
     OK = "OK"
     ERROR = "ERR"
+    SKIP = "SKIP"
 
 
 class PeerStatus(Enum):
@@ -50,9 +79,14 @@ class PeerStatus(Enum):
     UNRESPONSIVE = "unresponsive"
 
 
-PeerIdentity = bytes
-MessageDataType = TypeVar("MessageDataType")
-ResponseDataType = TypeVar("ResponseDataType")
+class ReceiveStatus(Enum):
+    """Status of the received method."""
+
+    OK = "OK"
+    INVALID_MESSAGE = "invalid"
+    CANCELLED = "cancelled"
+    SOCKET_CLOSED = "socket_closed"
+    TERMINATED = "terminated"
 
 
 def retry(
@@ -209,7 +243,7 @@ async def async_send_data(
 
 async def async_receive_data(
     reader: asyncio.StreamReader, size_bytes: int = 8
-) -> Optional[Tuple[PeerIdentity, Any]]:
+) -> Tuple[PeerIdentity, Any, Optional[bytes]]:
     """Receive data from the reader.
 
     The data is expected to be bytes-encoded JSON representation of a Python
@@ -225,7 +259,7 @@ async def async_receive_data(
     message_size = int.from_bytes(received, byteorder="big")
     received = await reader.readexactly(message_size)
     assert len(received) == message_size
-    return (b"", json.loads(received.decode("utf-8")))
+    return (b"", json.loads(received.decode("utf-8")), None)
 
 
 class Message(Generic[MessageDataType]):
@@ -238,6 +272,7 @@ class Message(Generic[MessageDataType]):
         message_data: MessageDataType,
         message_uuid: Optional[str] = None,
         sent_timestamp: Optional[float] = None,
+        client_id: Optional[bytes] = None,
     ):
         """Initialize.
 
@@ -248,6 +283,7 @@ class Message(Generic[MessageDataType]):
         self.type_data = type_data
         self.uuid = message_uuid or self._get_random_message_identifier()
         self.sent_timestamp = sent_timestamp
+        self.client_id = client_id
 
     def _get_random_message_identifier(self) -> str:
         """Get a random message identifier.
@@ -289,6 +325,12 @@ class Message(Generic[MessageDataType]):
             self.uuid,
         )
 
+    def respond_skip(
+        self, response_data: ResponseDataType
+    ) -> "Response[ResponseDataType]":
+        """Set the response status to skip."""
+        return self.respond(response_data, ResponseStatus.SKIP)
+
     def respond_ok(
         self, response_data: ResponseDataType
     ) -> "Response[ResponseDataType]":
@@ -306,6 +348,7 @@ class Message(Generic[MessageDataType]):
         command_name: str,
         message_data: MessageDataType,
         message_uuid: Optional[str] = None,
+        client_id: Optional[bytes] = None,
     ) -> "Message[MessageDataType]":
         """Construct and return a command."""
         return Message(
@@ -313,6 +356,7 @@ class Message(Generic[MessageDataType]):
             command_name,
             message_data,
             message_uuid,
+            client_id=client_id,
         )
 
     @staticmethod
@@ -362,6 +406,7 @@ class Message(Generic[MessageDataType]):
             message_dict["data"],
             message_dict.get("uuid"),
             message_dict.get("timestamp"),
+            message_dict.get("client_id"),
         )
 
     def to_dict(self) -> dict:
@@ -550,7 +595,9 @@ class BaseCommunicator:
         else:
             return call_command
 
-    async def _receive_message(self) -> Optional[Tuple[PeerIdentity, Message]]:
+    async def _receive_message(
+        self,
+    ) -> Tuple[ReceiveStatus, Optional[Tuple[PeerIdentity, Message]]]:
         """Receive a single message.
 
         This method is blocking: it waits for the message to arrive. The
@@ -568,6 +615,7 @@ class BaseCommunicator:
             the key "type".
         """
         result = None
+        received_status = ReceiveStatus.INVALID_MESSAGE
         try:
             self.logger.debug("Communicator %s waiting for message.", self.name)
             receive_task = asyncio.ensure_future(self.receive_method(self.reader))
@@ -579,25 +627,30 @@ class BaseCommunicator:
                 try:
                     received = receive_task.result()
                 except asyncio.IncompleteReadError:
+                    received_status = ReceiveStatus.SOCKET_CLOSED
                     self.logger.info("Socket closed by peer, stopping communication.")
                     received = None
                 if received is not None:
                     assert isinstance(received, tuple)
-                    assert len(received) == 2
+                    assert len(received) == 3
                     assert isinstance(received[0], bytes)
+                    received[1]["client_id"] = received[2]
                     assert Message.is_valid(received[1])
                     result = received[0], Message.from_dict(received[1])
+                    received_status = ReceiveStatus.OK
             else:
+                received_status = ReceiveStatus.TERMINATED
                 self.logger.debug(
                     "Communicator %s _receive_message: terminating flag is set, returning None",
                     self.name,
                 )
-        # Do not log cancelled errors.
         except asyncio.CancelledError:
+            received_status = ReceiveStatus.CANCELLED
             self.logger.debug(
                 "Communicator %s: CancelledError in _receive_message.", self.name
             )
         except:
+            received_status = ReceiveStatus.INVALID_MESSAGE
             self.logger.exception(
                 "Communicator %s: exception in _receive_message.", self.name
             )
@@ -605,7 +658,7 @@ class BaseCommunicator:
             # Always stop both tasks.
             receive_task.cancel()
             terminating_task.cancel()
-        return result
+        return received_status, result
 
     async def _send_message(
         self,
@@ -700,13 +753,17 @@ class BaseCommunicator:
         """
         try:
             while True:
-                received = await self._receive_message()
+                status, received = await self._receive_message()
 
                 if received is None:
-                    self.logger.info(
-                        f"Communicator {self.name}: received empty message, closing communicator."
-                    )
-                    break
+                    if status in [
+                        ReceiveStatus.SOCKET_CLOSED,
+                        ReceiveStatus.TERMINATED,
+                        ReceiveStatus.CANCELLED,
+                    ]:
+                        break
+                    elif ReceiveStatus.INVALID_MESSAGE:
+                        continue
 
                 identity, message = received
 
@@ -776,7 +833,7 @@ class BaseCommunicator:
         self,
         command: Message[MessageDataType],
         peer_identity: PeerIdentity = b"",
-        resend_timeout: Optional[int] = 30,
+        resend_timeout: Optional[int] = 60,
         timeout: Optional[int] = 1200,
     ) -> Response:
         """Send the command and return the response.
@@ -907,6 +964,7 @@ class BaseProtocol:
         communicator: BaseCommunicator,
         logger: logging.Logger,
         max_concurrent_commands: int = 10,
+        event_callback: Optional[MessageProcessingCallback] = None,
     ):
         """Initialize."""
         self.communicator = communicator
@@ -914,7 +972,17 @@ class BaseProtocol:
         self._should_stop = asyncio.Event()
         self._max_concurrent_commands = max_concurrent_commands
         self._concurrent_semaphore = asyncio.Semaphore(self._max_concurrent_commands)
-        self._command_counter = 0
+        self._event_callback = event_callback
+
+    def _call_event(
+        self,
+        event_type: MessageProcessingEventType,
+        message: Message,
+        peer_id: PeerIdentity,
+    ):
+        """Call the event callback."""
+        if self._event_callback is not None:
+            self._event_callback.event(event_type, message, peer_id)
 
     async def process_command(
         self, peer_identity: PeerIdentity, received_message: Message
@@ -924,12 +992,13 @@ class BaseProtocol:
         Use semaphore to make sure no more than max_concurrent_commands are
         processed at any given time.
         """
-        self.logger.debug(
-            "Concurrent semaphore count: %s.", self._concurrent_semaphore._value
-        )
-
         async with self._concurrent_semaphore:
             command_name = received_message.type_data
+            self._call_event(
+                MessageProcessingEventType.MESSAGE_PROCESSING_STARTED,
+                received_message,
+                peer_identity,
+            )
             handler_name = "handle_" + command_name
             handler = getattr(self, handler_name, None)
             if handler is None:
@@ -944,13 +1013,15 @@ class BaseProtocol:
                     f"Exception while running command handler {handler_name}: {ex}"
                 )
 
-            try:
-                await self.communicator.send_response(response, peer_identity)
-            except RuntimeError:
-                self.logger.exception(
-                    "Protocol: error sending response to {received_message}."
-                )
-                await self._abort_with_error("Error sending response.")
+            # Do not send response when status is set to SKIP.
+            if response.status != ResponseStatus.SKIP:
+                try:
+                    await self.communicator.send_response(response, peer_identity)
+                except RuntimeError:
+                    self.logger.exception(
+                        "Protocol: error sending response to {received_message}."
+                    )
+                    await self._abort_with_error("Error sending response.")
 
     def post_processing_command(
         self,
@@ -965,6 +1036,11 @@ class BaseProtocol:
         if handler is not None:
             self.logger.debug("Running post processing handler '%s'.", handler_name)
             asyncio.ensure_future(handler(received_message, peer_identity))
+        self._call_event(
+            MessageProcessingEventType.MESSAGE_PROCESSING_FINISHED,
+            received_message,
+            peer_identity,
+        )
 
     async def default_command_handler(
         self, message: Message, identity: PeerIdentity
@@ -1015,9 +1091,6 @@ class BaseProtocol:
                         ),
                         return_when=asyncio.FIRST_COMPLETED,
                     )
-                    self.logger.debug(
-                        "Communicate %s got nudged.", self.communicator.name
-                    )
 
                     if has_message_future in done:
                         self.logger.debug(
@@ -1028,6 +1101,11 @@ class BaseProtocol:
                                 peer_identity,
                                 received_command,
                             ) = await self.communicator.get_next_message()
+                            self._call_event(
+                                MessageProcessingEventType.MESSAGE_RECEIVED,
+                                received_command,
+                                peer_identity,
+                            )
                         except IndexError:
                             # On the next iteration the while loop will stop.
                             self.logger.exception(
