@@ -375,6 +375,50 @@ class RequirementsProcess(Process):
         print("Storage:", outputs.storage)
 
 
+class RequirementsProcess2(Process):
+    slug = "test-python-process-requirements2"
+    name = "Test Python Process Requirements 2"
+    version = "0.0.1"
+    process_type = "data:python:requirements"
+    requirements = {
+        "resources": {
+            "cores": 2,
+            "memory": 4096,
+            "storage": 200,
+        },
+    }
+
+    class Output:
+        """Input fields."""
+
+        cores = IntegerField(label="Cores")
+        memory = IntegerField(label="Memory")
+        storage = IntegerField(label="Storage")
+
+    def run(self, inputs, outputs):
+        outputs.cores = self.requirements["resources"]["cores"]
+        outputs.memory = self.requirements["resources"]["memory"]
+        outputs.storage = self.requirements["resources"]["storage"]
+
+
+class IterateProcess(Process):
+    slug = "test-python-process-iterate"
+    name = "Test iterating over filtered objects"
+    version = "0.0.1"
+    process_type = "data:python:iterate"
+
+    class Output:
+        """Input fields."""
+
+        process_slugs = ListField(
+            StringField(label="Slug of the process"),
+            label="Slugs of the processes.",
+        )
+
+    def run(self, inputs, outputs):
+        outputs.process_slugs = [process.slug for process in ProcessM.iterate()]
+
+
 class ProcessWithWorkflowInput(Process):
     slug = "process-with-workflow-input"
     name = "Process with workflow input"
@@ -628,23 +672,6 @@ class ChangeEntityName(Process):
         entity.name = inputs.entity_name
 
 
-class ChangeEntityDescriptor(Process):
-    slug = "change-entity-descriptor"
-    name = "Change entity descriptor"
-    data_name = "{{ data_input | name | default('?') }}"
-    version = "1.0.0"
-    process_type = "data:name"
-    requirements = {"expression-engine": "jinja"}
-
-    class Input:
-        entity_id = IntegerField(label="Entity id")
-        description = StringField(label="New description")
-
-    def run(self, inputs, outputs):
-        entity = Entity.get(pk=inputs.entity_id)
-        entity.descriptor = {"Description": inputs.description}
-
-
 class TestStorage(Process):
     slug = "storage-objects-test"
     name = "Test working with storage objects"
