@@ -1,15 +1,21 @@
 # pylint: disable=missing-docstring
+import math
 import os
 import sys
 import unittest
+import unittest.mock
+from unittest.mock import patch
 
 from django.test import LiveServerTestCase, override_settings
 
 import resolwe.permissions.models
+from resolwe.flow.managers.listener.plugin import (
+    ListenerPlugins,
+    listener_plugin_manager,
+)
 from resolwe.flow.models import (
     Collection,
     Data,
-    DescriptorSchema,
     Entity,
     Process,
     Relation,
@@ -24,6 +30,7 @@ from resolwe.flow.models.annotations import (
     AnnotationValue,
 )
 from resolwe.permissions.models import Permission, get_anonymous_user
+from resolwe.process import resources
 from resolwe.test import (
     ProcessTestCase,
     tag_process,
@@ -62,7 +69,6 @@ class PythonProcessTest(ProcessTestCase):
         self.assertEqual(process.description, "This is a process description.")
         self.assertEqual(process.data_name, "Foo: {{input_data | name}}")
         self.assertEqual(process.entity_type, "sample")
-        self.assertEqual(process.entity_descriptor_schema, "sample")
         self.assertEqual(process.entity_input, "input_data")
         self.assertEqual(
             process.requirements,
@@ -159,7 +165,9 @@ class PythonProcessTest(ProcessTestCase):
         # Now try reading and updating existing annotations.
         entity = Entity.objects.create(name="Entity", contributor=self.contributor)
         entity.set_permission(Permission.EDIT, self.contributor)
-        AnnotationValue.objects.create(entity=entity, field=age_field, value=42)
+        AnnotationValue.objects.create(
+            entity=entity, field=age_field, value=42, contributor=self.contributor
+        )
         process = Process.objects.get(
             slug="test-python-process-update-entity-annotations-v2"
         )
@@ -597,39 +605,6 @@ class PythonProcessTest(ProcessTestCase):
         entity.refresh_from_db()
         self.assertEqual(entity.name, "New entity name")
 
-    @with_docker_executor
-    @tag_process("change-entity-descriptor")
-    def test_change_descriptor(self):
-        """Assign descriptor to entity."""
-
-        descriptor_schema = DescriptorSchema.objects.create(
-            name="Descriptor schema",
-            contributor=self.contributor,
-            schema=[
-                {
-                    "name": "Description",
-                    "type": "basic:string:",
-                    "default": "default value",
-                }
-            ],
-        )
-        entity = Entity.objects.create(
-            name="Entity", contributor=self.user, descriptor_schema=descriptor_schema
-        )
-
-        self.run_process(
-            "change-entity-descriptor",
-            {
-                "entity_id": entity.pk,
-                "description": "New description",
-            },
-        )
-        entity.refresh_from_db()
-        self.assertEqual(
-            entity.descriptor,
-            {"Description": "New description"},
-        )
-
 
 class PythonProcessRequirementsTest(ProcessTestCase):
     def setUp(self):
@@ -688,6 +663,63 @@ class PythonProcessRequirementsTest(ProcessTestCase):
         self.assertEqual(data.output["memory"], 50000)
         self.assertEqual(data.output["storage"], 500)
 
+    @with_docker_executor
+    @tag_process("test-python-process-requirements2")
+    def test_dynamic_resources(self):
+        data = self.run_process("test-python-process-requirements2")
+        data.refresh_from_db()
+        self.assertEqual(data.status, "OK")
+
+        self.assertEqual(data.output["cores"], 2)
+        self.assertEqual(data.output["memory"], 4096)
+        self.assertEqual(data.output["storage"], 200)
+
+        @resources.estimator("test-python-process-requirements2", "cores")
+        def cores_estimator(data):
+            return 3
+
+        @resources.estimator("test-python-process-requirements2", "storage")
+        def storage_estimator(data):
+            return 3.5e9
+
+        data = self.run_process("test-python-process-requirements2")
+        data.refresh_from_db()
+        self.assertEqual(data.status, "OK")
+
+        self.assertEqual(data.output["cores"], 3)
+        self.assertEqual(data.output["memory"], 4096)  # default
+        self.assertEqual(data.output["storage"], 3.5e9)
+
+    @with_docker_executor
+    @tag_process("test-python-process-iterate")
+    def test_python_process_iterate(self):
+        """Test iteration in python processes."""
+        chunk_size = 2
+        with (
+            patch(
+                "resolwe.flow.managers.listener.python_process_plugin.MAX_CHUNK_SIZE",
+                chunk_size,
+            ),
+            patch.object(
+                ListenerPlugins,
+                "get_handler",
+                wraps=listener_plugin_manager.get_handler,
+            ) as plugin_mock,
+        ):
+            data = self.run_process("test-python-process-iterate")
+            calls = len(
+                [
+                    call
+                    for call in plugin_mock.call_args_list
+                    if call.args[0] == "iterate_objects"
+                ]
+            )
+            # Assert iterate method was called the correct number of times.
+            self.assertEqual(calls, math.ceil(Process.objects.count() / chunk_size))
+            # Assert the result is correct.
+            process_slugs = Process.objects.all().values_list("slug", flat=True)
+            self.assertCountEqual(process_slugs, data.output["process_slugs"])
+
 
 class PythonProcessDataBySlugTest(ProcessTestCase, LiveServerTestCase):
     def setUp(self):
