@@ -1,4 +1,5 @@
 """Google storage connector."""
+
 import base64
 import datetime
 import mimetypes
@@ -9,13 +10,19 @@ from pathlib import Path
 from google.api_core.exceptions import NotFound
 from google.cloud import storage
 
-from .baseconnector import BaseStorageConnector, validate_url, validate_urls
+from .baseconnector import (
+    BaseStorageConnector,
+    ConnectorType,
+    validate_url,
+    validate_urls,
+)
 
 
 class GoogleConnector(BaseStorageConnector):
     """Google Cloud Storage storage connector."""
 
     REQUIRED_SETTINGS = ["bucket", "credentials"]
+    CONNECTOR_TYPE = ConnectorType.GCS
 
     def __init__(self, config: dict, name: str):
         """Initialize Google connector."""
@@ -65,19 +72,17 @@ class GoogleConnector(BaseStorageConnector):
                         blob.delete()
 
     @validate_url
-    def push(self, stream, url, hash_type=None, data_hash=None):
+    def push(self, stream, url, chunk_size=BaseStorageConnector.CHUNK_SIZE, hashes={}):
         """Push data from the stream to the given URL."""
         url = os.fspath(url)
         mime_type = mimetypes.guess_type(url)[0]
         blob = self.bucket.blob(url)
-        if hash_type is not None:
-            assert hash_type in self.supported_hash
-            prop = self.hash_propery[hash_type]
-            setattr(blob, prop, data_hash)
         blob.upload_from_file(stream, content_type=mime_type)
+        if hashes:
+            self.set_hashes(url, hashes)
 
     @validate_url
-    def get(self, url, stream):
+    def get(self, url, stream, chunk_size=BaseStorageConnector.CHUNK_SIZE):
         """Get data from the given URL and write it into the given stream."""
         blob = self.bucket.blob(os.fspath(url))
         blob.download_to_file(stream)
