@@ -1,9 +1,15 @@
 """Local Storage connector."""
+
 import os
 import shutil
 from pathlib import Path
 
-from .baseconnector import BaseStorageConnector, validate_url, validate_urls
+from .baseconnector import (
+    BaseStorageConnector,
+    ConnectorType,
+    validate_url,
+    validate_urls,
+)
 from .hasher import StreamHasher
 
 
@@ -12,6 +18,7 @@ class LocalFilesystemConnector(BaseStorageConnector):
 
     #: Read files by chunks of the given size
     REQUIRED_SETTINGS = ["path"]
+    CONNECTOR_TYPE = ConnectorType.LOCAL
 
     def __init__(self, config: dict, name: str):
         """Connector initialization."""
@@ -48,8 +55,11 @@ class LocalFilesystemConnector(BaseStorageConnector):
                 shutil.rmtree(os.fspath(self.base_path / url))
 
     @validate_url
-    def push(self, stream, url, chunk_size=BaseStorageConnector.CHUNK_SIZE):
-        """Push data from the stream to the given URL."""
+    def push(self, stream, url, chunk_size=BaseStorageConnector.CHUNK_SIZE, hashes={}):
+        """Push data from the stream to the given URL.
+
+        The chunk_size and hashes arguments are ignored.
+        """
         data_remaining = True
         path = self.base_path / url
         path.parent.mkdir(parents=True, exist_ok=True)
@@ -166,6 +176,14 @@ class LocalFilesystemConnector(BaseStorageConnector):
         """Get a base path for this connector."""
         return Path(self.path)
 
+    @validate_url
+    def url(self, url):
+        """Return the URL to the given file in connector native notation.
+
+        For local connector this is the same as the presigned URL.
+        """
+        return self.presigned_url(url)
+
     @validate_url
     def presigned_url(self, url, expiration=3600, force_download=False):
         """Create a presigned URL.
