@@ -13,7 +13,6 @@ from urllib.parse import urlencode, urlparse
 
 import pytz
 from bs4 import BeautifulSoup
-
 from django.core.exceptions import ImproperlyConfigured, PermissionDenied
 from django.http import HttpRequest, HttpResponse, JsonResponse
 from django.shortcuts import redirect
@@ -22,15 +21,14 @@ from django.utils import timezone
 from django.utils.decorators import method_decorator
 from django.views import View
 from django.views.decorators.csrf import csrf_exempt
-
 from rest_framework import serializers
 from rest_framework.response import Response
-from rest_framework.viewsets import ViewSet
+from rest_framework.views import APIView
 
 from resolwe.flow.models import Data
 from resolwe.flow.models.collection import Collection
 from resolwe.permissions.utils import get_user
-from resolwe.storage.connectors import connectors
+from resolwe.storage.connectors import ConnectorType, connectors
 from resolwe.storage.connectors.baseconnector import BaseStorageConnector
 from resolwe.storage.models import FileStorage, ReferencedPath
 from resolwe.test.utils import ignore_in_tests
@@ -45,12 +43,54 @@ class UploadConfigSerializer(serializers.Serializer):
     config = serializers.DictField()
 
 
-class UploadConfig(ViewSet):
+class RFC3339DateTimeField(serializers.DateTimeField):
+    """DateTime field that serializes to RFC3339 format."""
+
+    format = "%Y-%m-%dT%H:%M:%SZ"
+
+
+class UploadCredentialsSerializer(serializers.Serializer):
+    """Serializer for upload credentials."""
+
+    AccessKeyId = serializers.CharField()
+    SecretAccessKey = serializers.CharField()
+    Token = serializers.CharField()
+    Expiration = RFC3339DateTimeField()
+
+
+class UploadCredentials(APIView):
+    """Get the upload credentials."""
+
+    # Used to generate the API schema.
+    serializer_class = UploadCredentialsSerializer
+
+    def get(self, request, *args, **kwargs):
+        """Return the upload credentials."""
+        try:
+            upload_connector = connectors.for_storage("upload")[0]
+        except Exception:
+            message = "Upload connector could not be determined."
+            logger.exception(message)
+            raise ImproperlyConfigured(message)
+
+        if upload_connector.CONNECTOR_TYPE != ConnectorType.S3:
+            raise ImproperlyConfigured(
+                "Credentials endpoint only supports S3 connector."
+            )
+        prefix = str(get_user(request.user).id)
+        credentials = upload_connector.temporary_credentials(prefix)["credentials"]
+        # The credentials are returned in the format expected by the AWS SDK, except
+        # the key "SessionToken", which must be transformed to "Token".
+        credentials["Token"] = credentials.pop("SessionToken")
+        return Response(self.serializer_class(credentials).data)
+
+
+class UploadConfig(APIView):
     """Get the upload configuration."""
 
     serializer_class = UploadConfigSerializer
 
-    def list(self, request):
+    def get(self, request, *args, **kwargs):
         """Return the JSON representing the upload configuration.
 
         The returning object is JSON representation of the dictionary with the
