@@ -7,11 +7,10 @@ import json
 import os
 import sys
 from distutils.util import strtobool
+from importlib import metadata
 from pathlib import Path
 
-from decouple import Csv, config
-
-from resolwe.__about__ import __version__
+from decouple import config
 
 PROJECT_ROOT = Path(__file__).parent.resolve()
 
@@ -36,7 +35,6 @@ INSTALLED_APPS = (
     "rest_framework",
     "drf_spectacular",
     "django_filters",
-    "versionfield",
     "resolwe",
     "resolwe.permissions",
     "resolwe.flow",
@@ -123,6 +121,20 @@ LISTENER_CONNECTION = {
     ),
 }
 
+# The number of data points to send in one request.
+# See
+# https://github.com/open-telemetry/opentelemetry-python/pull/2809
+# and
+# https://github.com/open-telemetry/opentelemetry-python/issues/2710
+# for details.
+FLOW_METRICS_EXPORT_SIZE = config(
+    "RESOLWE_METRICS_EXPORT_SIZE", cast=int, default=1000
+)
+
+# The endpoint to use for listener metrics.
+FLOW_METRICS_ENDPOINT = config("RESOLWE_METRICS_ENDPOINT", default=None)
+
+
 # The IP address where listener is available from the communication container.
 # The setting is a dictionary where key is the name of the workload connector.
 COMMUNICATION_CONTAINER_LISTENER_CONNECTION = config(
@@ -256,6 +268,9 @@ REST_FRAMEWORK = {
         "resolwe.permissions.filters.ResolwePermissionsFilter",
         "django_filters.rest_framework.DjangoFilterBackend",
         "resolwe.flow.filters.OrderingFilter",
+        # Full text search filter must be the last one in the list so it can override
+        # the default order specified by the ordering filter.
+        "resolwe.flow.filters.FullTextSearchFilter",
     ),
     "EXCEPTION_HANDLER": "resolwe.flow.utils.exceptions.resolwe_exception_handler",
     "DATETIME_FORMAT": "iso-8601",
@@ -269,7 +284,7 @@ REST_FRAMEWORK = {
 SPECTACULAR_SETTINGS = {
     "TITLE": "Resolwe",
     "DESCRIPTION": "An open source dataflow package for Django.",
-    "VERSION": __version__,
+    "VERSION": metadata.metadata("resolwe")["version"],
     "SERVE_INCLUDE_SCHEMA": False,
     "ENUM_NAME_OVERRIDES": {"DataStatus": "resolwe.flow.models.Data.STATUS_CHOICES"},
 }
@@ -312,11 +327,11 @@ TEST_PROCESS_PROFILE = False
 # logging debugging messages to to a file.
 debug_file_path = os.environ.get("RESOLWE_LOG_FILE", os.devnull)
 
-github_actions = os.environ.get("GITHUB_ACTIONS") == "true"
+github_actions_debug = strtobool(os.environ.get("GITHUB_ACTIONS_DEBUG", "0"))
 CONSOLE_LEVEL = "WARNING"
 default_logger_handlers = ["file"]
 
-if github_actions:
+if github_actions_debug:
     CONSOLE_LEVEL = "DEBUG"
     default_logger_handlers = ["console", "file"]
 
@@ -328,7 +343,7 @@ LOGGING = {
             "format": "%(asctime)s - %(levelname)s - %(name)s[%(process)s]: %(message)s",
         },
         "auditlog": {
-            "format": "%(asctime)s - %(name)s[%(process)s]: %(message)s ; SID='%(session_id)s' RID='%(request_id)s'",
+            "format": "%(asctime)s - %(name)s[%(process)s]: %(message)s ; UID='%(user_id)s' SID='%(session_id)s' RID='%(request_id)s'",
         },
     },
     "handlers": {
